@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2022 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2023 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * https://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -28,43 +28,38 @@ class CPPNamespaceDeclaration;
 
 #include "dsymbol.h"
 
-enum
+enum class CSX : uint16_t
 {
-    CSXthis_ctor  = 1,      // called this()
-    CSXsuper_ctor = 2,      // called super()
-    CSXthis       = 4,      // referenced this
-    CSXsuper      = 8,      // referenced super
-    CSXlabel      = 0x10,   // seen a label
-    CSXreturn     = 0x20,   // seen a return statement
-    CSXany_ctor   = 0x40,   // either this() or super() was called
-    CSXhalt       = 0x80,   // assert(0)
+    none       = 0,
+    this_ctor  = 1,      // called this()
+    super_ctor = 2,      // called super()
+    label      = 4,      // seen a label
+    return_    = 8,      // seen a return statement
+    any_ctor   = 0x10,   // either this() or super() was called
+    halt       = 0x20,   // assert(0)
 };
 
-enum
+enum class SCOPE
 {
     // Flags that would not be inherited beyond scope nesting
-    SCOPEctor          = 0x0001,  // constructor type
-    SCOPEcondition     = 0x0004,  // inside static if/assert condition
-    SCOPEdebug         = 0x0008,  // inside debug conditional
+    ctor          = 0x0001,  // constructor type
+    noaccesscheck = 0x0002,  // don't do access checks
+    condition     = 0x0004,  // inside static if/assert condition
+    debug_        = 0x0008,  // inside debug conditional
 
     // Flags that would be inherited beyond scope nesting
-    SCOPEnoaccesscheck = 0x0002,  // don't do access checks
-    SCOPEconstraint    = 0x0010,  // inside template constraint
-    SCOPEinvariant     = 0x0020,  // inside invariant code
-    SCOPErequire       = 0x0040,  // inside in contract code
-    SCOPEensure        = 0x0060,  // inside out contract code
-    SCOPEcontract      = 0x0060,  // [mask] we're inside contract code
-    SCOPEctfe          = 0x0080,  // inside a ctfe-only expression
-    SCOPEcompile       = 0x0100,  // inside __traits(compile)
-    SCOPEignoresymbolvisibility = 0x0200,  // ignore symbol visibility (Bugzilla 15907)
+    constraint    = 0x0010,  // inside template constraint
+    invariant_    = 0x0020,  // inside invariant code
+    require       = 0x0040,  // inside in contract code
+    ensure        = 0x0060,  // inside out contract code
+    contract      = 0x0060,  // [mask] we're inside contract code
+    ctfe          = 0x0080,  // inside a ctfe-only expression
+    compile       = 0x0100,  // inside __traits(compile)
+    ignoresymbolvisibility = 0x0200,  // ignore symbol visibility (Bugzilla 15907)
 
-    SCOPEfree          = 0x8000,  // is on free list
-    SCOPEfullinst      = 0x10000, // fully instantiate templates
-    SCOPEalias         = 0x20000, // inside alias declaration
-
-    // The following are mutually exclusive
-    SCOPEprintf        = 0x40000, // printf-style function
-    SCOPEscanf         = 0x80000, // scanf-style function
+    Cfile         = 0x0800,  // C semantics apply
+    free          = 0x8000,  // is on free list
+    fullinst      = 0x10000, // fully instantiate templates
 };
 
 struct Scope
@@ -74,6 +69,7 @@ struct Scope
     Module *_module;            // Root module
     ScopeDsymbol *scopesym;     // current symbol
     FuncDeclaration *func;      // function we are in
+    VarDeclaration  *varDecl;   // variable we are in during semantic2
     Dsymbol *parent;            // parent to use
     LabelStatement *slabel;     // enclosing labelled statement
     SwitchStatement *sw;        // enclosing switch statement
@@ -85,8 +81,8 @@ struct Scope
     ForeachStatement *fes;      // if nested function for ForeachStatement, this is it
     Scope *callsc;              // used for __FUNCTION__, __PRETTY_FUNCTION__ and __MODULE__
     Dsymbol *inunion;           // !=null if processing members of a union
-    bool nofree;                // true if shouldn't free it
-    bool inLoop;                // true if inside a loop (where constructor calls aren't allowed)
+    d_bool nofree;                // true if shouldn't free it
+    d_bool inLoop;                // true if inside a loop (where constructor calls aren't allowed)
     int intypeof;               // in typeof(exp)
     VarDeclaration *lastVar;    // Previous symbol used to prevent goto-skips-init
 
@@ -98,8 +94,8 @@ struct Scope
     Module *minst;              // root module where the instantiated templates should belong to
     TemplateInstance *tinst;    // enclosing template instance
 
-    unsigned char callSuper;    // primitive flow analysis for constructors
-    unsigned char *fieldinit;
+    CSX callSuper;              // primitive flow analysis for constructors
+    CSX *fieldinit;
     size_t fieldinit_dim;
 
     AlignDeclaration *aligndecl;    // alignment for struct members
@@ -131,23 +127,6 @@ struct Scope
 
     AliasDeclaration *aliasAsg; // if set, then aliasAsg is being assigned a new value,
                                 // do not set wasRead for it
-    Scope();
-
-    Scope *copy();
-
-    Scope *push();
-    Scope *push(ScopeDsymbol *ss);
-    Scope *pop();
-
-    Scope *startCTFE();
-    Scope *endCTFE();
 
     Dsymbol *search(const Loc &loc, Identifier *ident, Dsymbol **pscopesym, int flags = IgnoreNone);
-
-    ClassDeclaration *getClassScope();
-    AggregateDeclaration *getStructClassScope();
-
-    structalign_t alignment();
-
-    bool isDeprecated() const;
 };
