@@ -20,9 +20,7 @@
 #include "llvm/Object/MachO.h"
 #include "llvm/Object/ObjectFile.h"
 #include "llvm/Support/Errc.h"
-#if LDC_LLVM_VER >= 1100
 #include "llvm/Support/Host.h"
-#endif
 #include "llvm/Support/Path.h"
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/ToolDrivers/llvm-lib/LibDriver.h"
@@ -352,7 +350,7 @@ int createStaticLibrary() {
   }
 
   // invoke external archiver
-  return executeToolAndWait(tool, args, global.params.verbose);
+  return executeToolAndWait(Loc(), tool, args, global.params.verbose);
 }
 
 const char *getPathToProducedStaticLibrary() {
