@@ -28,11 +28,7 @@
 #if LDC_LLVM_VER >= 1400
 #include "llvm/IR/DiagnosticInfo.h"
 #endif
-#if LDC_LLVM_VER >= 1100
 #include "llvm/IR/LLVMRemarkStreamer.h"
-#else
-#include "llvm/IR/RemarkStreamer.h"
-#endif
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/Path.h"
 #include "llvm/Support/ToolOutputFile.h"
@@ -66,13 +62,8 @@ createAndSetDiagnosticsOutputFile(IRState &irs, llvm::LLVMContext &ctx,
     // If there is instrumentation data available, also output function hotness
     const bool withHotness = opts::isUsingPGOProfile();
 
-    auto remarksFileOrError =
-#if LDC_LLVM_VER >= 1100
-        llvm::setupLLVMOptimizationRemarks(
-#else
-        llvm::setupOptimizationRemarks(
-#endif
-            ctx, diagnosticsFilename, "", "", withHotness);
+    auto remarksFileOrError = llvm::setupLLVMOptimizationRemarks(
+        ctx, diagnosticsFilename, "", "", withHotness);
     if (llvm::Error e = remarksFileOrError.takeError()) {
       irs.dmodule->error("Could not create file %s: %s",
                          diagnosticsFilename.c_str(),
@@ -161,8 +152,13 @@ bool inlineAsmDiagnostic(IRState *irs, const llvm::SMDiagnostic &d,
 #if LDC_LLVM_VER < 1300
 void inlineAsmDiagnosticHandler(const llvm::SMDiagnostic &d, void *context,
                                 unsigned locCookie) {
-  if (d.getKind() == llvm::SourceMgr::DK_Error)
+  if (d.getKind() == llvm::SourceMgr::DK_Error) {
     ++global.errors;
+  } else if (global.params.warnings == DIAGNOSTICerror &&
+             d.getKind() == llvm::SourceMgr::DK_Warning) {
+    ++global.warnings;
+  }
+
   inlineAsmDiagnostic(static_cast<IRState *>(context), d, locCookie);
 }
 #else
@@ -172,12 +168,20 @@ struct InlineAsmDiagnosticHandler : public llvm::DiagnosticHandler {
 
     // return false to defer to LLVMContext::diagnose()
   bool handleDiagnostics(const llvm::DiagnosticInfo &DI) override {
+    if (DI.getKind() == llvm::SourceMgr::DK_Error ||
+        DI.getSeverity() == llvm::DS_Error) {
+      ++global.errors;
+    } else if (global.params.warnings == DIAGNOSTICerror &&
+               (DI.getKind() == llvm::SourceMgr::DK_Warning ||
+                DI.getSeverity() == llvm::DS_Warning)) {
+      ++global.warnings;
+    }
+
     if (DI.getKind() != llvm::DK_SrcMgr)
         return false;
 
     const auto &DISM = llvm::cast<llvm::DiagnosticInfoSrcMgr>(DI);
-    if (DISM.getKind() == llvm::SourceMgr::DK_Error)
-      ++global.errors;
+
     return inlineAsmDiagnostic(irs, DISM.getSMDiag(), DISM.getLocCookie());
   }
 };
@@ -198,7 +202,7 @@ CodeGenerator::CodeGenerator(llvm::LLVMContext &context,
       moduleCount_(0), singleObj_(singleObj), ir_(nullptr) {
   // Set the context to discard value names when not generating textual IR and
   // when ASan or MSan are not enabled.
-  if (!global.params.output_ll &&
+  if (!global.params.output_ll && !opts::fNoDiscardValueNames &&
       !opts::isSanitizerEnabled(opts::AddressSanitizer |
                                 opts::MemorySanitizer)) {
     context_.setDiscardValueNames(true);
@@ -364,15 +368,24 @@ void CodeGenerator::writeMLIRModule(mlir::OwningModuleRef *module,
     const auto llpath = replaceExtensionWith(mlir_ext, filename);
     Logger::println("Writting MLIR to %s\n", llpath.c_str());
     std::error_code errinfo;
-    llvm::raw_fd_ostream aos(llpath, errinfo, llvm::sys::fs::OF_None);
+    llvm::ToolOutputFile aos(llpath, errinfo, llvm::sys::fs::OF_None);
 
-    if (aos.has_error()) {
+    if (aos.os().has_error()) {
       error(Loc(), "Cannot write MLIR file '%s': %s", llpath.c_str(),
             errinfo.message().c_str());
       fatal();
     }
 
     // module->print(aos);
+
+    // Terminate upon errors during the LLVM passes.
+    if (global.errors || global.warnings) {
+      Logger::println(
+          "Aborting because of errors/warnings during bitcode LLVM passes");
+      fatal();
+    }
+
+    aos.keep();
   }
 }
 
