@@ -195,7 +195,11 @@ static std::vector<std::string> getDefaultLibNames() {
 llvm::Optional<std::vector<std::string>> getExplicitPlatformLibs() {
   if (platformLib.getNumOccurrences() > 0)
     return parseLibNames(platformLib);
+#if LDC_LLVM_VER < 1600
   return llvm::None;
+#else
+  return std::nullopt;
+#endif
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -323,7 +327,7 @@ int runProgram() {
 
   // Run executable
   int status =
-      executeToolAndWait(gExePath, opts::runargs, global.params.verbose);
+      executeToolAndWait(Loc(), gExePath, opts::runargs, global.params.verbose);
   if (status < 0) {
 #if defined(_MSC_VER) || defined(__MINGW32__)
     error(Loc(), "program received signal %d", -status);
