@@ -23,11 +23,13 @@
 #include "dmd/target.h"
 #include "driver/args.h"
 #include "driver/cache.h"
+#include "driver/cl_helpers.h"
 #include "driver/cl_options.h"
 #include "driver/cl_options_instrumentation.h"
 #include "driver/cl_options_sanitizers.h"
 #include "driver/codegenerator.h"
 #include "driver/configfile.h"
+#include "driver/cpreprocessor.h"
 #include "driver/dcomputecodegenerator.h"
 #include "driver/exe_path.h"
 #include "driver/ldc-version.h"
@@ -35,8 +37,7 @@
 #include "driver/plugins.h"
 #include "driver/targetmachine.h"
 #include "driver/timetrace.h"
-#include "gen/abi.h"
-#include "gen/cl_helpers.h"
+#include "gen/abi/abi.h"
 #include "gen/irstate.h"
 #include "gen/ldctraits.h"
 #include "gen/linkage.h"
@@ -100,6 +101,9 @@ static cl::opt<bool> enableGC(
     cl::desc("Enable the garbage collector for the LDC front-end. This reduces "
              "the compiler memory requirements but increases compile times."));
 
+// Deprecated options
+extern llvm::cl::opt<bool> checkPrintf;
+
 namespace {
 
 // This function exits the program.
@@ -339,40 +343,40 @@ void parseCommandLine(Strings &sourceFiles) {
   global.params.objname = opts::fromPathString(objectFile);
   global.params.objdir = opts::fromPathString(objectDir);
 
-  global.params.docdir = opts::fromPathString(ddocDir);
-  global.params.docname = opts::fromPathString(ddocFile);
-  global.params.doDocComments |=
-      global.params.docdir.length || global.params.docname.length;
+  global.params.ddoc.dir = opts::fromPathString(ddocDir);
+  global.params.ddoc.name = opts::fromPathString(ddocFile);
+  global.params.ddoc.doOutput |=
+      global.params.ddoc.dir.length || global.params.ddoc.name.length;
 
-  global.params.jsonfilename = opts::fromPathString(jsonFile);
-  if (global.params.jsonfilename.length || !jsonFields.empty()) {
-    global.params.doJsonGeneration = true;
+  global.params.json.name = opts::fromPathString(jsonFile);
+  if (global.params.json.name.length || !jsonFields.empty()) {
+    global.params.json.doOutput = true;
   }
 
-  global.params.hdrdir = opts::fromPathString(hdrDir);
-  global.params.hdrname = opts::fromPathString(hdrFile);
-  global.params.doHdrGeneration |=
-      global.params.hdrdir.length || global.params.hdrname.length;
+  global.params.dihdr.dir = opts::fromPathString(hdrDir);
+  global.params.dihdr.name = opts::fromPathString(hdrFile);
+  global.params.dihdr.doOutput |=
+      global.params.dihdr.dir.length || global.params.dihdr.name.length;
 
-  global.params.cxxhdrdir = opts::fromPathString(cxxHdrDir);
-  global.params.cxxhdrname = opts::fromPathString(cxxHdrFile);
-  if (global.params.doCxxHdrGeneration == CxxHeaderMode::none &&
-      (global.params.cxxhdrdir.length || global.params.cxxhdrname.length)) {
-    global.params.doCxxHdrGeneration = CxxHeaderMode::silent;
-  }
+  global.params.cxxhdr.dir = opts::fromPathString(cxxHdrDir);
+  global.params.cxxhdr.name = opts::fromPathString(cxxHdrFile);
+  global.params.cxxhdr.doOutput |=
+      global.params.cxxhdr.dir.length || global.params.cxxhdr.name.length;
 
-  global.params.mixinFile = opts::fromPathString(mixinFile).ptr;
+  global.params.mixinOut.name = opts::fromPathString(mixinFile);
+  global.params.mixinOut.doOutput |= global.params.mixinOut.name.length;
 
   if (moduleDeps.getNumOccurrences() != 0) {
-    global.params.moduleDeps = createOutBuffer();
+    global.params.moduleDeps.doOutput = true;
+    global.params.moduleDeps.buffer = createOutBuffer();
     if (!moduleDeps.empty())
-      global.params.moduleDepsFile = opts::dupPathString(moduleDeps);
+      global.params.moduleDeps.name = opts::dupPathString(moduleDeps);
   }
 
   if (makeDeps.getNumOccurrences() != 0) {
-    global.params.emitMakeDeps = true;
+    global.params.makeDeps.doOutput = true;
     if (!makeDeps.empty())
-      global.params.makeDepsFile = opts::dupPathString(makeDeps);
+      global.params.makeDeps.name = opts::dupPathString(makeDeps);
   }
 
 #if _WIN32
@@ -436,10 +440,18 @@ void parseCommandLine(Strings &sourceFiles) {
   if (global.params.useDIP1000 == FeatureState::enabled) // DIP1000 implies DIP25
     global.params.useDIP25 = FeatureState::enabled;
   // legacy -dip25 option
-  if (global.params.useDIP25 == FeatureState::default_ &&
-      opts::useDIP25.getNumOccurrences()) {
-    global.params.useDIP25 =
-        opts::useDIP25 ? FeatureState::enabled : FeatureState::disabled;
+  if (opts::useDIP25.getNumOccurrences()) {
+    deprecation(Loc(), "`-dip25` no longer has any effect");
+  }
+
+  // -betterC implies -allinst (since D v2.105)
+  if (global.params.betterC) {
+    global.params.allInst = true;
+  }
+
+  // -wo implies at least -wi (print the warnings)
+  if (global.params.obsolete && global.params.warnings == DIAGNOSTICoff) {
+    global.params.warnings = DIAGNOSTICinform;
   }
 
   global.params.output_o =
@@ -541,8 +553,29 @@ void parseCommandLine(Strings &sourceFiles) {
     error(Loc(), "-soname can be used only when building a shared library");
   }
 
-  global.params.hdrStripPlainFunctions = !opts::hdrKeepAllBodies;
+  global.params.dihdr.fullOutput = opts::hdrKeepAllBodies;
   global.params.disableRedZone = opts::disableRedZone();
+
+  // Passmanager selection options depend on LLVM version
+#if LDC_LLVM_VER < 1400
+  // LLVM < 14 only supports the legacy passmanager
+  if (!opts::isUsingLegacyPassManager()) {
+    error(Loc(), "LLVM version 13 or below only supports --passmanager=legacy");
+  }
+#endif
+#if LDC_LLVM_VER >= 1500
+  // LLVM >= 15 only supports the new passmanager
+  if (opts::isUsingLegacyPassManager()) {
+    error(Loc(), "LLVM version 15 or above only supports --passmanager=new");
+  }
+#endif
+
+#if LDC_LLVM_VER >= 1500
+  getGlobalContext().setOpaquePointers(opts::enableOpaqueIRPointers);
+#elif LDC_LLVM_VER >= 1400
+  if (opts::enableOpaqueIRPointers)
+    getGlobalContext().enableOpaquePointers();
+#endif
 }
 
 void initializePasses() {
@@ -552,11 +585,18 @@ void initializePasses() {
   initializeCore(Registry);
   initializeTransformUtils(Registry);
   initializeScalarOpts(Registry);
+#if LDC_LLVM_VER < 1600
   initializeObjCARCOpts(Registry);
+#endif
   initializeVectorization(Registry);
   initializeInstCombine(Registry);
+#if LDC_LLVM_VER < 1600
+  initializeAggressiveInstCombine(Registry);
+#endif
   initializeIPO(Registry);
+#if LDC_LLVM_VER < 1600
   initializeInstrumentation(Registry);
+#endif
   initializeAnalysis(Registry);
   initializeCodeGen(Registry);
   initializeGlobalISel(Registry);
@@ -725,6 +765,16 @@ void registerPredefinedTargetVersions() {
   case llvm::Triple::wasm64:
     VersionCondition::addPredefinedGlobalIdent("WebAssembly");
     break;
+#if LDC_LLVM_VER >= 1600
+  case llvm::Triple::loongarch32:
+    VersionCondition::addPredefinedGlobalIdent("LoongArch32");
+    registerPredefinedFloatABI("LoongArch_SoftFloat", "LoongArch_HardFloat");
+    break;
+  case llvm::Triple::loongarch64:
+    VersionCondition::addPredefinedGlobalIdent("LoongArch64");
+    registerPredefinedFloatABI("LoongArch_SoftFloat", "LoongArch_HardFloat");
+    break;
+#endif // LDC_LLVM_VER >= 1600
   default:
     warning(Loc(), "unknown target CPU architecture: %s",
             triple.getArchName().str().c_str());
@@ -883,6 +933,11 @@ void registerPredefinedTargetVersions() {
     }
     break;
   }
+
+  //Issue deprecations for deprecated arguments
+  // N.B `-nodefaultlib` is checked in `getDefaultLibNames`
+  if (checkPrintf)
+    deprecation(Loc(), "'-check-printf-calls' is deprecated, use `pragma(printf)` instead.");
 }
 
 } // anonymous namespace
@@ -900,7 +955,7 @@ void registerPredefinedVersions() {
   }
 #endif
 
-  if (global.params.doDocComments) {
+  if (global.params.ddoc.doOutput) {
     VersionCondition::addPredefinedGlobalIdent("D_Ddoc");
   }
 
@@ -944,6 +999,10 @@ void registerPredefinedVersions() {
     VersionCondition::addPredefinedGlobalIdent("D_ProfileGC");
   }
 
+  if (isOptimizationEnabled()) {
+    VersionCondition::addPredefinedGlobalIdent("D_Optimized");
+  }
+
   registerPredefinedTargetVersions();
 
   // `D_ObjectiveC` is added by the dmd.objc.Supported ctor
@@ -966,6 +1025,13 @@ void registerPredefinedVersions() {
     VersionCondition::addPredefinedGlobalIdent("LDC_ThreadSanitizer");
   }
 
+#if LDC_LLVM_VER >= 1400
+  // A version identifier for whether opaque pointers are enabled or not. (needed e.g. for intrinsic mangling)
+  if (!getGlobalContext().supportsTypedPointers()) {
+    VersionCondition::addPredefinedGlobalIdent("LDC_LLVM_OpaquePointers");
+  }
+#endif
+
 // Expose LLVM version to runtime
 #define STR(x) #x
 #define XSTR(x) STR(x)
@@ -1068,6 +1134,11 @@ int cppmain() {
     fatal();
   }
 
+  global.compileEnv.previewIn = global.params.previewIn;
+  global.compileEnv.ddocOutput = global.params.ddoc.doOutput;
+  global.compileEnv.shortenedMethods = global.params.shortenedMethods;
+  global.compileEnv.obsolete = global.params.obsolete;
+
   if (opts::fTimeTrace) {
     initializeTimeTrace(opts::fTimeTraceGranularity, 0, opts::allArguments[0]);
   }
@@ -1090,9 +1161,15 @@ int cppmain() {
   }
 
   auto relocModel = getRelocModel();
+#if LDC_LLVM_VER >= 1600
+  if (global.params.dll && !relocModel.has_value()) {
+    relocModel = llvm::Reloc::PIC_;
+  }
+#else
   if (global.params.dll && !relocModel.hasValue()) {
     relocModel = llvm::Reloc::PIC_;
   }
+#endif
 
   fixupTripleEnv(mTargetTriple);
 
@@ -1134,6 +1211,8 @@ int cppmain() {
     global.params.dllimport = DLLImport::none;
   }
 
+  global.preprocess = &runCPreprocessor;
+
   // allocate the target abi
   gABI = TargetABI::getTarget();
 
