@@ -44,6 +44,13 @@
 namespace cl = llvm::cl;
 using LLMetadata = llvm::Metadata;
 
+#if LDC_LLVM_VER >= 1600
+namespace llvm {
+  template <typename T> using Optional = std::optional<T>;
+  inline constexpr std::nullopt_t None = std::nullopt;
+}
+#endif
+
 static cl::opt<cl::boolOrDefault> emitColumnInfo(
     "gcolumn-info", cl::ZeroOrMore, cl::Hidden,
     cl::desc("Include column numbers in line debug infos. Defaults to "
@@ -109,7 +116,7 @@ DIBuilder::DIBuilder(IRState *const IR)
       emitColumnInfo(opts::getFlagOrDefault(::emitColumnInfo, !emitCodeView)) {}
 
 unsigned DIBuilder::getColumn(const Loc &loc) const {
-  return (loc.linnum && emitColumnInfo) ? loc.charnum : 0;
+  return (loc.linnum() && emitColumnInfo) ? loc.charnum() : 0;
 }
 
 // Returns the DI scope of a symbol.
@@ -195,7 +202,7 @@ llvm::StringRef DIBuilder::GetNameAndScope(Dsymbol *sym, DIScope &scope) {
 // Sets the memory address for a debuginfo variable.
 void DIBuilder::Declare(const Loc &loc, llvm::Value *storage,
                         DILocalVariable divar, DIExpression diexpr) {
-  auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum,
+  auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum(),
                                         getColumn(loc), GetCurrentScope());
   DBuilder.insertDeclare(storage, divar, diexpr, debugLoc, IR->scopebb());
 }
@@ -203,7 +210,7 @@ void DIBuilder::Declare(const Loc &loc, llvm::Value *storage,
 // Sets the (current) value for a debuginfo variable.
 void DIBuilder::SetValue(const Loc &loc, llvm::Value *value,
                          DILocalVariable divar, DIExpression diexpr) {
-  auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum,
+  auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum(),
                                         getColumn(loc), GetCurrentScope());
   DBuilder.insertDbgValueIntrinsic(value, divar, diexpr, debugLoc,
                                    IR->scopebb());
@@ -233,13 +240,13 @@ DIFile DIBuilder::CreateFile(const char *filename) {
 }
 
 DIFile DIBuilder::CreateFile(const Loc &loc) {
-  return CreateFile(loc.filename);
+  return CreateFile(loc.filename());
 }
 
 DIFile DIBuilder::CreateFile(Dsymbol *decl) {
   const char *filename = nullptr;
   for (Dsymbol *sym = decl; sym && !filename; sym = sym->parent)
-    filename = sym->loc.filename;
+    filename = sym->loc.filename();
   return CreateFile(filename);
 }
 
@@ -339,7 +346,7 @@ DIType DIBuilder::CreateEnumType(TypeEnum *type) {
 
   DIScope scope = nullptr;
   const auto name = GetNameAndScope(ed, scope);
-  const auto lineNumber = ed->loc.linnum;
+  const auto lineNumber = ed->loc.linnum();
   const auto file = CreateFile(ed);
 
   // just emit a typedef for non-integral base types
@@ -385,12 +392,8 @@ DIType DIBuilder::CreateVectorType(TypeVector *type) {
   LLType *T = DtoType(type);
 
   const auto dim = type->basetype->isTypeSArray()->dim->toInteger();
-#if LDC_LLVM_VER >= 1100
   const auto Dim = llvm::ConstantAsMetadata::get(DtoConstSize_t(dim));
   auto subscript = DBuilder.getOrCreateSubrange(Dim, nullptr, nullptr, nullptr);
-#else
-  auto subscript = DBuilder.getOrCreateSubrange(0, dim);
-#endif
 
   return DBuilder.createVectorType(
       getTypeAllocSize(T) * 8, // size (bits)
@@ -487,7 +490,7 @@ void DIBuilder::AddFields(AggregateDeclaration *ad, DIFile file,
     if (vd->type->toBasetype()->isTypeNoreturn())
       continue;
 
-    elems.push_back(CreateMemberType(vd->loc.linnum, vd->type, file,
+    elems.push_back(CreateMemberType(vd->loc.linnum(), vd->type, file,
                                      vd->toChars(), vd->offset,
                                      vd->visibility.kind));
   }
@@ -513,21 +516,35 @@ void DIBuilder::AddStaticMembers(AggregateDeclaration *ad, DIFile file,
         // Also DWARF supports imported declarations, but LLVM
         // currently does nothing with DIImportedEntity except at CU-level.
         visitMembers(tmixin->members);
-      } else if (auto vd = s->isVarDeclaration())
-        if (vd->isDataseg() && !vd->aliassym /* TODO: tuples*/ &&
-            !vd->type->toBasetype()->isTypeNoreturn()) {
-          llvm::MDNode *elem =
-              CreateMemberType(vd->loc.linnum, vd->type, file, vd->toChars(), 0,
-                               vd->visibility.kind,
-                               /*isStatic = */ true, scope);
-          elems.push_back(elem);
-          StaticDataMemberCache[vd].reset(elem);
+      } else if (auto vd = s->isVarDeclaration()) {
+        if (vd->isDataseg()) {
+          if (auto td = vd->aliasTuple) { // ugly kludge for tuples
+            if (td->isexp && td->objects) {
+              Dsymbols tupleVars;
+              for (auto o : *td->objects) {
+                if (auto e = isExpression(o))
+                  if (auto ve = e->isVarExp())
+                    if (auto vd2 = ve->var->isVarDeclaration())
+                      if (vd2->isDataseg())
+                        tupleVars.push(vd2);
+              }
+              visitMembers(&tupleVars);
+            }
+          } else if (!vd->type->toBasetype()->isTypeNoreturn()) {
+            llvm::MDNode *elem =
+                CreateMemberType(vd->loc.linnum(), vd->type, file,
+                                 vd->toChars(), 0, vd->visibility.kind,
+                                 /*isStatic = */ true, scope);
+            elems.push_back(elem);
+            StaticDataMemberCache[vd].reset(elem);
+          }
         }
-    } /*else if (auto fd = s->isFuncDeclaration())*/ // Clang also adds static
-                                                     // functions as
-                                                     // declarations, but they
-                                                     // already work without
-                                                     // adding them.
+      } /*else if (auto fd = s->isFuncDeclaration())*/ // Clang also adds static
+                                                       // functions as
+                                                       // declarations, but they
+                                                       // already work without
+                                                       // adding them.
+    }
   };
   visitMembers(members);
 }
@@ -545,10 +562,9 @@ DIType DIBuilder::CreateCompositeType(Type *t) {
 
   // Use the actual type associated with the declaration, ignoring any
   // const/wrappers.
-  LLType *T = DtoType(ad->type);
-  if (t->ty == TY::Tclass)
-    T = T->getPointerElementType();
+  DtoType(ad->type);
   IrAggr *irAggr = getIrAggr(ad, true);
+  LLType *T = irAggr->getLLStructType();
 
   if (irAggr->diCompositeType) {
     return irAggr->diCompositeType;
@@ -570,7 +586,7 @@ DIType DIBuilder::CreateCompositeType(Type *t) {
 
   // defaults
   const auto file = CreateFile(ad);
-  const auto lineNum = ad->loc.linnum;
+  const auto lineNum = ad->loc.linnum();
   const auto sizeInBits = T->isSized() ? getTypeAllocSize(T) * 8 : 0;
   const auto alignmentInBits = T->isSized() ? getABITypeAlign(T) * 8 : 0;
   const auto classOffsetInBits = 0;
@@ -656,13 +672,9 @@ DIType DIBuilder::CreateSArrayType(TypeSArray *type) {
   for (; te->ty == TY::Tsarray; te = te->nextOf()) {
     TypeSArray *tsa = static_cast<TypeSArray *>(te);
     const auto count = tsa->dim->toInteger();
-#if LDC_LLVM_VER >= 1100
     const auto Count = llvm::ConstantAsMetadata::get(DtoConstSize_t(count));
     const auto subscript =
         DBuilder.getOrCreateSubrange(Count, nullptr, nullptr, nullptr);
-#else
-    const auto subscript = DBuilder.getOrCreateSubrange(0, count);
-#endif
     subscripts.push_back(subscript);
   }
 
@@ -896,9 +908,9 @@ void DIBuilder::EmitImport(Import *im) {
   auto diModule = EmitModule(im->mod);
 
   DBuilder.createImportedModule(GetCurrentScope(),
-                                diModule,       // imported module
-                                CreateFile(im), // file
-                                im->loc.linnum  // line num
+                                diModule,        // imported module
+                                CreateFile(im),  // file
+                                im->loc.linnum() // line num
   );
 }
 
@@ -933,7 +945,7 @@ DISubprogram DIBuilder::EmitSubProgram(FuncDeclaration *fd) {
 
   const auto linkageName = irFunc->getLLVMFuncName();
   const auto file = CreateFile(fd);
-  const auto lineNo = fd->loc.linnum;
+  const auto lineNo = fd->loc.linnum();
   const auto isLocalToUnit = fd->visibility.kind == Visibility::private_;
   const auto isDefinition = true;
   const auto scopeLine = lineNo; // FIXME
@@ -999,7 +1011,7 @@ DISubprogram DIBuilder::EmitThunk(llvm::Function *Thunk, FuncDeclaration *fd) {
   const auto name = (llvm::Twine(fd->toChars()) + ".__thunk").str();
   const auto linkageName = Thunk->getName();
   const auto file = CreateFile(fd);
-  const auto lineNo = fd->loc.linnum;
+  const auto lineNo = fd->loc.linnum();
   const bool isLocalToUnit = fd->visibility.kind == Visibility::private_;
   const bool isDefinition = true;
   const bool isOptimized = isOptimizationEnabled();
@@ -1068,7 +1080,7 @@ void DIBuilder::EmitBlockStart(const Loc &loc) {
   LOG_SCOPE;
 
   DILexicalBlock block = DBuilder.createLexicalBlock(
-      GetCurrentScope(), CreateFile(loc), loc.linnum, getColumn(loc));
+      GetCurrentScope(), CreateFile(loc), loc.linnum(), getColumn(loc));
   IR->func()->diLexicalBlocks.push(block);
   EmitStopPoint(loc);
 }
@@ -1092,14 +1104,14 @@ void DIBuilder::EmitStopPoint(const Loc &loc) {
   // If we already have a location set and the current loc is invalid
   // (line 0), then we can just ignore it (see GitHub issue #998 for why we
   // cannot do this in all cases).
-  if (!loc.linnum && IR->ir->getCurrentDebugLocation())
+  if (!loc.linnum() && IR->ir->getCurrentDebugLocation())
     return;
 
-  unsigned linnum = loc.linnum;
+  unsigned linnum = loc.linnum();
   // without proper loc use the line of the enclosing symbol that has line
   // number debug info
   for (Dsymbol *sym = IR->func()->decl; sym && !linnum; sym = sym->parent)
-    linnum = sym->loc.linnum;
+    linnum = sym->loc.linnum();
   if (!linnum)
     linnum = 1;
 
@@ -1204,7 +1216,7 @@ void DIBuilder::EmitLocalVariable(llvm::Value *ll, VarDeclaration *vd,
   const auto scope = GetCurrentScope();
   const auto name = vd->toChars();
   const auto file = CreateFile(vd);
-  const auto lineNum = vd->loc.linnum;
+  const auto lineNum = vd->loc.linnum();
   const auto preserve = true;
   auto flags = !isThisPtr
                    ? DIFlags::FlagZero
@@ -1276,12 +1288,10 @@ void DIBuilder::EmitGlobalVariable(llvm::GlobalVariable *llVar,
       vd->toChars(),                         // name
       mangleBuf.peekChars(),                 // linkage name
       CreateFile(vd),                        // file
-      vd->loc.linnum,                        // line num
+      vd->loc.linnum(),                        // line num
       CreateTypeDescription(vd->type),       // type
       vd->visibility.kind == Visibility::private_, // is local to unit
-#if LDC_LLVM_VER >= 1000
       !(vd->storage_class & STCextern),      // bool isDefined
-#endif
       nullptr,                               // DIExpression *Expr
       Decl                                   // declaration
   );
