@@ -17,9 +17,8 @@
 #include "dmd/init.h"
 #include "dmd/module.h"
 #include "dmd/template.h"
-#include "gen/abi.h"
+#include "gen/abi/abi.h"
 #include "gen/arrays.h"
-#include "gen/cl_helpers.h"
 #include "gen/classes.h"
 #include "gen/complex.h"
 #include "gen/dvalue.h"
@@ -40,6 +39,7 @@
 #include "ir/irfunction.h"
 #include "ir/irmodule.h"
 #include "ir/irtypeaggr.h"
+#include "ir/irtypeclass.h"
 #include "llvm/MC/MCAsmInfo.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/ManagedStatic.h"
@@ -91,15 +91,6 @@ LLValue *DtoNew(const Loc &loc, Type *newtype) {
   return DtoBitCast(mem, DtoPtrToType(newtype), ".gc_mem");
 }
 
-LLValue *DtoNewStruct(const Loc &loc, TypeStruct *newtype) {
-  llvm::Function *fn = getRuntimeFunction(
-      loc, gIR->module,
-      newtype->isZeroInit(newtype->sym->loc) ? "_d_newitemT" : "_d_newitemiT");
-  LLConstant *ti = DtoTypeInfoOf(loc, newtype);
-  LLValue *mem = gIR->CreateCallOrInvoke(fn, ti, ".gc_struct");
-  return DtoBitCast(mem, DtoPtrToType(newtype), ".gc_struct");
-}
-
 void DtoDeleteMemory(const Loc &loc, DValue *ptr) {
   llvm::Function *fn = getRuntimeFunction(loc, gIR->module, "_d_delmemory");
   LLValue *lval = (ptr->isLVal() ? DtoLVal(ptr) : makeLValue(loc, ptr));
@@ -191,7 +182,7 @@ llvm::AllocaInst *DtoArrayAlloca(Type *type, unsigned arraysize,
       lltype, gIR->module.getDataLayout().getAllocaAddrSpace(),
       DtoConstUint(arraysize), name, gIR->topallocapoint());
   if (auto alignment = DtoAlignment(type)) {
-    ai->setAlignment(LLAlign(alignment));
+    ai->setAlignment(llvm::Align(alignment));
   }
   return ai;
 }
@@ -202,22 +193,11 @@ llvm::AllocaInst *DtoRawAlloca(LLType *lltype, size_t alignment,
       lltype, gIR->module.getDataLayout().getAllocaAddrSpace(), name,
       gIR->topallocapoint());
   if (alignment) {
-    ai->setAlignment(LLAlign(alignment));
+    ai->setAlignment(llvm::Align(alignment));
   }
   return ai;
 }
 
-LLValue *DtoGcMalloc(const Loc &loc, LLType *lltype, const char *name) {
-  // get runtime function
-  llvm::Function *fn = getRuntimeFunction(loc, gIR->module, "_d_allocmemory");
-  // parameters
-  LLValue *size = DtoConstSize_t(getTypeAllocSize(lltype));
-  // call runtime allocator
-  LLValue *mem = gIR->CreateCallOrInvoke(fn, size, name);
-  // cast
-  return DtoBitCast(mem, getPtrToType(lltype), name);
-}
-
 LLValue *DtoAllocaDump(DValue *val, const char *name) {
   return DtoAllocaDump(val, val->type, name);
 }
@@ -237,7 +217,7 @@ LLValue *DtoAllocaDump(DValue *val, LLType *asType, int alignment,
     LLType *asMemType = i1ToI8(voidToI8(asType));
     LLValue *copy = DtoRawAlloca(asMemType, alignment, name);
     const auto minSize =
-        std::min(getTypeAllocSize(lval->getType()->getPointerElementType()),
+        std::min(getTypeAllocSize(DtoType(val->type)),
                  getTypeAllocSize(asMemType));
     const auto minAlignment =
         std::min(DtoAlignment(val->type), static_cast<unsigned>(alignment));
@@ -290,7 +270,7 @@ void DtoAssert(Module *M, const Loc &loc, DValue *msg) {
   args.push_back(DtoModuleFileName(M, loc));
 
   // line param
-  args.push_back(DtoConstUint(loc.linnum));
+  args.push_back(DtoConstUint(loc.linnum()));
 
   // call
   gIR->CreateCallOrInvoke(fn, args);
@@ -302,8 +282,8 @@ void DtoAssert(Module *M, const Loc &loc, DValue *msg) {
 void DtoCAssert(Module *M, const Loc &loc, LLValue *msg) {
   const auto &triple = *global.params.targetTriple;
   const auto file =
-      DtoConstCString(loc.filename ? loc.filename : M->srcfile.toChars());
-  const auto line = DtoConstUint(loc.linnum);
+      DtoConstCString(loc.filename() ? loc.filename() : M->srcfile.toChars());
+  const auto line = DtoConstUint(loc.linnum());
   const auto fn = getCAssertFunction(loc, gIR->module);
 
   llvm::SmallVector<LLValue *, 4> args;
@@ -328,6 +308,14 @@ void DtoCAssert(Module *M, const Loc &loc, LLValue *msg) {
     args.push_back(file);
     args.push_back(line);
     args.push_back(msg);
+  } else if (global.params.isNewlibEnvironment) {
+    const auto irFunc = gIR->func();
+    const auto funcName =
+        irFunc && irFunc->decl ? irFunc->decl->toPrettyChars() : "";
+    args.push_back(file);
+    args.push_back(line);
+    args.push_back(DtoConstCString(funcName));
+    args.push_back(msg);
   } else {
     args.push_back(msg);
     args.push_back(file);
@@ -359,7 +347,7 @@ void DtoThrow(const Loc &loc, DValue *e) {
  ******************************************************************************/
 
 LLConstant *DtoModuleFileName(Module *M, const Loc &loc) {
-  return DtoConstString(loc.filename ? loc.filename : M->srcfile.toChars());
+  return DtoConstString(loc.filename() ? loc.filename() : M->srcfile.toChars());
 }
 
 /******************************************************************************
@@ -397,6 +385,11 @@ void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
     return;
   }
 
+  if (auto bfLVal = lhs->isBitFieldLVal()) {
+    bfLVal->store(DtoRVal(rhs));
+    return;
+  }
+
   if (t->ty == TY::Tbool) {
     DtoStoreZextI8(DtoRVal(rhs), DtoLVal(lhs));
   } else if (t->ty == TY::Tstruct) {
@@ -409,7 +402,7 @@ void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
       // time as to not emit an invalid (overlapping) memcpy on trivial
       // struct self-assignments like 'A a; a = a;'.
       if (src != dst)
-        DtoMemCpy(dst, src);
+        DtoMemCpy(DtoType(lhs->type), dst, src);
     }
   } else if (t->ty == TY::Tarray || t->ty == TY::Tsarray) {
     DtoArrayAssign(loc, lhs, rhs, op, canSkipPostblit);
@@ -429,7 +422,7 @@ void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
       Logger::cout() << "l : " << *l << '\n';
       Logger::cout() << "r : " << *r << '\n';
     }
-    r = DtoBitCast(r, l->getType()->getContainedType(0));
+    r = DtoBitCast(r, DtoType(lhs->type));
     DtoStore(r, l);
   } else if (t->iscomplex()) {
     LLValue *dst = DtoLVal(lhs);
@@ -442,7 +435,7 @@ void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
       Logger::cout() << "lhs: " << *l << '\n';
       Logger::cout() << "rhs: " << *r << '\n';
     }
-    LLType *lit = l->getType()->getContainedType(0);
+    LLType *lit = DtoType(lhs->type);
     if (r->getType() != lit) {
       r = DtoRVal(DtoCast(loc, rhs, lhs->type));
       IF_LOG {
@@ -842,9 +835,9 @@ void DtoResolveVariable(VarDeclaration *vd) {
   // just forward aliases
   // TODO: Is this required here or is the check in VarDeclaration::codegen
   // sufficient?
-  if (vd->aliassym) {
-    Logger::println("alias sym");
-    DtoResolveDsymbol(vd->aliassym);
+  if (vd->aliasTuple) {
+    Logger::println("aliasTuple");
+    DtoResolveDsymbol(vd->aliasTuple);
     return;
   }
 
@@ -878,7 +871,7 @@ void DtoResolveVariable(VarDeclaration *vd) {
 void DtoVarDeclaration(VarDeclaration *vd) {
   assert(!vd->isDataseg() &&
          "Statics/globals are handled in DtoDeclarationExp.");
-  assert(!vd->aliassym && "Aliases are handled in DtoDeclarationExp.");
+  assert(!vd->aliasTuple && "Aliases are handled in DtoDeclarationExp.");
 
   IF_LOG Logger::println("DtoVarDeclaration(vdtype = %s)", vd->type->toChars());
   LOG_SCOPE
@@ -917,19 +910,29 @@ void DtoVarDeclaration(VarDeclaration *vd) {
     Type *type = isSpecialRefVar(vd) ? vd->type->pointerTo() : vd->type;
 
     llvm::Value *allocainst;
+    bool isRealAlloca = false;
     LLType *lltype = DtoType(type); // void for noreturn
     if (lltype->isVoidTy() || gDataLayout->getTypeSizeInBits(lltype) == 0) {
       allocainst = llvm::ConstantPointerNull::get(getPtrToType(lltype));
     } else if (type != vd->type) {
       allocainst = DtoAlloca(type, vd->toChars());
+      isRealAlloca = true;
     } else {
       allocainst = DtoAlloca(vd, vd->toChars());
+      isRealAlloca = true;
     }
 
     irLocal->value = allocainst;
 
     if (!lltype->isVoidTy())
       gIR->DBuilder.EmitLocalVariable(allocainst, vd);
+
+    // Lifetime annotation is only valid on alloca.
+    if (isRealAlloca) {
+      // The lifetime of a stack variable starts from the point it is declared
+      gIR->funcGen().localVariableLifetimeAnnotator.addLocalVariable(
+          allocainst, DtoConstUlong(type->size()));
+    }
   }
 
   IF_LOG Logger::cout() << "llvm value for decl: " << *getIrLocal(vd)->value
@@ -951,11 +954,11 @@ DValue *DtoDeclarationExp(Dsymbol *declaration) {
   if (VarDeclaration *vd = declaration->isVarDeclaration()) {
     Logger::println("VarDeclaration");
 
-    // if aliassym is set, this VarDecl is redone as an alias to another symbol
+    // if aliasTuple is set, this VarDecl is redone as an alias to another symbol
     // this seems to be done to rewrite Tuple!(...) v;
     // as a TupleDecl that contains a bunch of individual VarDecls
-    if (vd->aliassym) {
-      return DtoDeclarationExp(vd->aliassym);
+    if (vd->aliasTuple) {
+      return DtoDeclarationExp(vd->aliasTuple);
     }
 
     if (vd->storage_class & STCmanifest) {
@@ -1019,7 +1022,7 @@ LLValue *DtoRawVarDeclaration(VarDeclaration *var, LLValue *addr) {
   assert(!var->isDataseg());
 
   // we don't handle aliases either
-  assert(!var->aliassym);
+  assert(!var->aliasTuple);
 
   IrLocal *irLocal = isIrLocalCreated(var) ? getIrLocal(var) : nullptr;
 
@@ -1176,7 +1179,7 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
       val = llvm::ConstantArray::get(at, elements);
     }
 
-    (void)numTotalVals;
+    (void)numTotalVals; (void) product; // Silence unused variable warning when assert is disabled.
     assert(product == numTotalVals);
     return val;
   }
@@ -1190,10 +1193,8 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
         static_cast<TypeSArray *>(tv->basetype)->dim->toInteger();
 #if LDC_LLVM_VER >= 1200
     const auto elementCount = llvm::ElementCount::getFixed(elemCount);
-#elif LDC_LLVM_VER >= 1100
-    const auto elementCount = llvm::ElementCount(elemCount, false);
 #else
-    const auto elementCount = elemCount;
+    const auto elementCount = llvm::ElementCount(elemCount, false);
 #endif
     return llvm::ConstantVector::getSplat(elementCount, val);
   }
@@ -1269,11 +1270,7 @@ static char *DtoOverloadedIntrinsicName(TemplateInstance *ti,
   if (dtype->isPPC_FP128Ty()) { // special case
     replacement = "ppcf128";
   } else if (dtype->isVectorTy()) {
-#if LDC_LLVM_VER >= 1100
     auto vectorType = llvm::cast<llvm::FixedVectorType>(dtype);
-#else
-    auto vectorType = llvm::cast<llvm::VectorType>(dtype);
-#endif
     llvm::raw_string_ostream stream(replacement);
     stream << 'v' << vectorType->getNumElements() << prefix
            << gDataLayout->getTypeSizeInBits(vectorType->getElementType());
@@ -1321,24 +1318,6 @@ void DtoSetFuncDeclIntrinsicName(TemplateInstance *ti, TemplateDeclaration *td,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-size_t getMemberSize(Type *type) {
-  const dinteger_t dSize = type->size();
-  llvm::Type *const llType = DtoType(type);
-  if (!llType->isSized()) {
-    // Forward reference in a cycle or similar, we need to trust the D type.
-    return dSize;
-  }
-
-  const uint64_t llSize = gDataLayout->getTypeAllocSize(llType);
-  assert(llSize <= dSize &&
-         "LLVM type is bigger than the corresponding D type, "
-         "might lead to aggregate layout mismatch.");
-
-  return llSize;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-
 Type *stripModifiers(Type *type, bool transitive) {
   if (type->ty == TY::Tfunction) {
     return type;
@@ -1610,14 +1589,14 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
       assert(tb->ty == TY::Tarray && tb->nextOf()->ty == TY::Tvoid);
       const auto size = DtoConstSize_t(ad->structsize);
       llvm::Constant *ptr =
-          sd && sd->zeroInit
+          sd && sd->zeroInit()
               ? getNullValue(getVoidPtrType())
               : DtoBitCast(getIrAggr(ad)->getInitSymbol(), getVoidPtrType());
       return new DSliceValue(type, size, ptr);
     }
 
     assert(sd);
-    if (sd->zeroInit) {
+    if (sd->zeroInit()) {
       error(loc, "no init symbol for zero-initialized struct");
       fatal();
     }
@@ -1630,14 +1609,6 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
 }
 
 llvm::Constant *DtoConstSymbolAddress(const Loc &loc, Declaration *decl) {
-  // Make sure 'this' isn't needed.
-  // TODO: This check really does not belong here, should be moved to
-  // semantic analysis in the frontend.
-  if (decl->needThis()) {
-    error(loc, "need `this` to access `%s`", decl->toChars());
-    fatal();
-  }
-
   // global variable
   if (VarDeclaration *vd = decl->isVarDeclaration()) {
     if (!vd->isDataseg()) {
@@ -1668,30 +1639,31 @@ llvm::Constant *DtoConstSymbolAddress(const Loc &loc, Declaration *decl) {
   llvm_unreachable("Taking constant address not implemented.");
 }
 
-llvm::Constant *buildStringLiteralConstant(StringExp *se, bool zeroTerm) {
-  if (se->sz == 1) {
+llvm::Constant *buildStringLiteralConstant(StringExp *se,
+                                           uint64_t bufferLength) {
+  const auto stringLength = se->numberOfCodeUnits();
+  assert(bufferLength >= stringLength);
+
+  if (se->sz == 1 && bufferLength <= stringLength + 1) {
     const DString data = se->peekString();
+    const bool nullTerminate = (bufferLength == stringLength + 1);
     return llvm::ConstantDataArray::getString(
-        gIR->context(), {data.ptr, data.length}, zeroTerm);
+        gIR->context(), {data.ptr, data.length}, nullTerminate);
   }
 
   Type *dtype = se->type->toBasetype();
   Type *cty = dtype->nextOf()->toBasetype();
-
   LLType *ct = DtoMemType(cty);
-  auto len = se->numberOfCodeUnits();
-  if (zeroTerm) {
-    len += 1;
-  }
-  LLArrayType *at = LLArrayType::get(ct, len);
+  LLArrayType *at = LLArrayType::get(ct, bufferLength);
 
   std::vector<LLConstant *> vals;
-  vals.reserve(len);
-  for (size_t i = 0; i < se->numberOfCodeUnits(); ++i) {
+  vals.reserve(bufferLength);
+  for (uint64_t i = 0; i < stringLength; ++i) {
     vals.push_back(LLConstantInt::get(ct, se->getCodeUnit(i), false));
   }
-  if (zeroTerm) {
-    vals.push_back(LLConstantInt::get(ct, 0, false));
+  const auto nullChar = LLConstantInt::get(ct, 0, false);
+  for (uint64_t i = stringLength; i < bufferLength; ++i) {
+    vals.push_back(nullChar);
   }
   return LLConstantArray::get(at, vals);
 }
@@ -1878,7 +1850,7 @@ FuncDeclaration *getParentFunc(Dsymbol *sym) {
   return nullptr;
 }
 
-LLValue *DtoIndexAggregate(LLValue *src, AggregateDeclaration *ad,
+DLValue *DtoIndexAggregate(LLValue *src, AggregateDeclaration *ad,
                            VarDeclaration *vd) {
   IF_LOG Logger::println("Indexing aggregate field %s:", vd->toPrettyChars());
   LOG_SCOPE;
@@ -1889,48 +1861,54 @@ LLValue *DtoIndexAggregate(LLValue *src, AggregateDeclaration *ad,
   DtoResolveDsymbol(ad);
 
   // Look up field to index or offset to apply.
-  unsigned fieldIndex;
-  unsigned byteOffset;
   auto irTypeAggr = getIrType(ad->type)->isAggr();
   assert(irTypeAggr);
-  irTypeAggr->getMemberLocation(vd, fieldIndex, byteOffset);
+  bool isFieldIdx;
+  unsigned off = irTypeAggr->getMemberLocation(vd, isFieldIdx);
 
-  LLValue *val = src;
-  if (byteOffset) {
-    assert(fieldIndex == 0);
+  LLValue *ptr = src;
+  LLType * ty = nullptr;
+  if (!isFieldIdx) {
     // Cast to void* to apply byte-wise offset from object start.
-    val = DtoBitCast(val, getVoidPtrType());
-    val = DtoGEP1(val, byteOffset);
+    ptr = DtoBitCast(ptr, getVoidPtrType());
+    ptr = DtoGEP1(llvm::Type::getInt8Ty(gIR->context()), ptr, off);
+    ty = DtoType(vd->type);
   } else {
     if (ad->structsize == 0) { // can happen for extern(C) structs
-      assert(fieldIndex == 0);
+      assert(off == 0);
     } else {
       // Cast the pointer we got to the canonical struct type the indices are
       // based on.
-      LLType *st = DtoType(ad->type);
-      if (ad->isStructDeclaration()) {
-        st = getPtrToType(st);
+      LLType *st = nullptr;
+      if (auto irtc = irTypeAggr->isClass()) {
+        st = irtc->getMemoryLLType();
+      } else {
+        st = irTypeAggr->getLLType();
       }
-      val = DtoBitCast(val, st);
-      val = DtoGEP(val, 0, fieldIndex);
+      ptr = DtoBitCast(ptr, st->getPointerTo());
+      ptr = DtoGEP(st, ptr, 0, off);
+      ty = isaStruct(st)->getElementType(off);
     }
   }
 
   // Cast the (possibly void*) pointer to the canonical variable type.
-  val = DtoBitCast(val, DtoPtrToType(vd->type));
+  ptr = DtoBitCast(ptr, DtoPtrToType(vd->type));
 
-  IF_LOG Logger::cout() << "Value: " << *val << '\n';
-  return val;
+  IF_LOG Logger::cout() << "Pointer: " << *ptr << '\n';
+  if (auto p = isaPointer(ty)) {
+    if (p->getAddressSpace())
+      return new DDcomputeLValue(vd->type, p, ptr);
+  }
+  return new DLValue(vd->type, ptr);
 }
 
 unsigned getFieldGEPIndex(AggregateDeclaration *ad, VarDeclaration *vd) {
-  unsigned fieldIndex;
-  unsigned byteOffset;
   auto irTypeAggr = getIrType(ad->type)->isAggr();
   assert(irTypeAggr);
-  irTypeAggr->getMemberLocation(vd, fieldIndex, byteOffset);
-  assert(byteOffset == 0 && "Cannot address field by a simple GEP.");
-  return fieldIndex;
+  bool isFieldIdx;
+  unsigned off = irTypeAggr->getMemberLocation(vd, isFieldIdx);
+  assert(isFieldIdx && "Cannot address field by a simple GEP.");
+  return off;
 }
 
 DValue *makeVarDValue(Type *type, VarDeclaration *vd, llvm::Value *storage) {
@@ -1947,6 +1925,7 @@ DValue *makeVarDValue(Type *type, VarDeclaration *vd, llvm::Value *storage) {
     expectedType = expectedType->getPointerTo();
 
   if (val->getType() != expectedType) {
+#if LDC_LLVM_VER < 1500
     // The type of globals is determined by their initializer, and the front-end
     // may inject implicit casts for class references and static arrays.
     assert(vd->isDataseg() || (vd->storage_class & STCextern) ||
@@ -1959,6 +1938,7 @@ DValue *makeVarDValue(Type *type, VarDeclaration *vd, llvm::Value *storage) {
     // work as well.
     assert(getTypeStoreSize(DtoType(type)) <= getTypeStoreSize(pointeeType) &&
            "LValue type mismatch, encountered type too small.");
+#endif
     val = DtoBitCast(val, expectedType);
   }
 
