@@ -24,7 +24,7 @@
 #include "dmd/template.h"
 #include "driver/cl_options_instrumentation.h"
 #include "driver/timetrace.h"
-#include "gen/abi.h"
+#include "gen/abi/abi.h"
 #include "gen/arrays.h"
 #include "gen/functions.h"
 #include "gen/irstate.h"
@@ -268,7 +268,8 @@ void addCoverageAnalysis(Module *m) {
                                                init, "_d_cover_data");
 
     d_cover_data_slice = DtoConstSlice(DtoConstSize_t(m->numlines),
-                                       DtoGEP(m->d_cover_data, 0, 0));
+                                       DtoGEP(m->d_cover_data->getValueType(),
+                                              m->d_cover_data, 0, 0));
   }
 
   // Create "static constructor" that calls _d_cover_register2(string filename,
@@ -293,7 +294,11 @@ void addCoverageAnalysis(Module *m) {
     ctor->setCallingConv(gABI->callingConv(LINK::d));
     // Set function attributes. See functions.cpp:DtoDefineFunction()
     if (global.params.targetTriple->getArch() == llvm::Triple::x86_64) {
+#if LDC_LLVM_VER >= 1500
+      ctor->setUWTableKind(llvm::UWTableKind::Default);
+#else
       ctor->addFnAttr(LLAttribute::UWTable);
+#endif
     }
 
     llvm::BasicBlock *bb = llvm::BasicBlock::Create(gIR->context(), "", ctor);
@@ -388,8 +393,27 @@ void registerModuleInfo(Module *m) {
     emitModuleRefToSection(mangle, moduleInfoSym);
   }
 }
+
+void addModuleFlags(llvm::Module &m) {
+#if LDC_LLVM_VER >= 1500
+  const auto ModuleMinFlag = llvm::Module::Min;
+#else
+  const auto ModuleMinFlag = llvm::Module::Warning; // Fallback value
+#endif
+
+  if (opts::fCFProtection == opts::CFProtectionType::Return ||
+      opts::fCFProtection == opts::CFProtectionType::Full) {
+    m.addModuleFlag(ModuleMinFlag, "cf-protection-return", 1);
+  }
+
+  if (opts::fCFProtection == opts::CFProtectionType::Branch ||
+      opts::fCFProtection == opts::CFProtectionType::Full) {
+    m.addModuleFlag(ModuleMinFlag, "cf-protection-branch", 1);
+  }
 }
 
+} // anonymous namespace
+
 void codegenModule(IRState *irs, Module *m) {
   TimeTraceScope timeScope("Generate IR", m->toChars(), m->loc);
 
@@ -428,9 +452,11 @@ void codegenModule(IRState *irs, Module *m) {
 
   // Skip emission of all the additional module metadata if:
   // a) the -betterC switch is on,
-  // b) requested explicitly by the user via pragma(LDC_no_moduleinfo), or if
-  // c) there's no ModuleInfo declaration.
-  if (global.params.useModuleInfo && !m->noModuleInfo && Module::moduleinfo) {
+  // b) requested explicitly by the user via pragma(LDC_no_moduleinfo),
+  // c) there's no ModuleInfo declaration, or if
+  // d) the module is a C file.
+  if (global.params.useModuleInfo && !m->noModuleInfo && Module::moduleinfo &&
+      m->filetype != FileType::c) {
     // generate ModuleInfo
     registerModuleInfo(m);
   }
@@ -439,6 +465,8 @@ void codegenModule(IRState *irs, Module *m) {
     addCoverageAnalysisInitializer(m);
   }
 
+  addModuleFlags(irs->module);
+
   gIR = nullptr;
   irs->dmodule = nullptr;
 }
