@@ -17,6 +17,7 @@
 #include "gen/irstate.h"
 #include "gen/llvmhelpers.h"
 #include "gen/logger.h"
+#include "gen/runtime.h"
 #include "gen/tollvm.h"
 #include "ir/irfunction.h"
 #include "ir/irtypeaggr.h"
@@ -36,6 +37,23 @@ bool isNRVOVar(VarDeclaration *vd) {
 bool captureByRef(VarDeclaration *vd) {
   return vd->isReference() || isNRVOVar(vd);
 }
+LLValue *loadThisPtr(AggregateDeclaration *ad, IrFunction &irfunc) {
+  if (ad->isClassDeclaration()) {
+      return DtoLoad(DtoType(irfunc.irFty.arg_this->type),
+                     irfunc.thisArg);
+  }
+
+  return irfunc.thisArg;
+}
+
+LLValue *indexVThis(AggregateDeclaration *ad,  LLValue* val) {
+  DtoResolveDsymbol(ad);
+  llvm::StructType *st = getIrAggr(ad)->getLLStructType();
+  unsigned idx = getVthisIdx(ad);
+  return DtoLoad(st->getElementType(idx),
+                 DtoGEP(st, val, 0, idx, ".vthis"));
+}
+
 } // anonymous namespace
 
 static void DtoCreateNestedContextType(FuncDeclaration *fd);
@@ -80,11 +98,11 @@ DValue *DtoNestedVariable(const Loc &loc, Type *astype, VarDeclaration *vd,
   } else if (AggregateDeclaration *ad = irfunc->decl->isMember2()) {
     Logger::println(
         "Current function is member of nested class, loading vthis");
-    LLValue *val =
-        ad->isClassDeclaration() ? DtoLoad(irfunc->thisArg) : irfunc->thisArg;
+    LLValue *val = loadThisPtr(ad, *irfunc);
+
     for (; ad; ad = ad->toParent2()->isAggregateDeclaration()) {
       assert(ad->vthis);
-      val = DtoLoad(DtoGEP(val, 0, getVthisIdx(ad), ".vthis"));
+      val = indexVThis(ad, val);
     }
     ctx = val;
     skipDIDeclaration = true;
@@ -118,10 +136,10 @@ DValue *DtoNestedVariable(const Loc &loc, Type *astype, VarDeclaration *vd,
   // Extract variable from nested context
 
   assert(irfunc->frameType);
-  const auto frameType = LLPointerType::getUnqual(irfunc->frameType);
+  const auto pframeType = LLPointerType::getUnqual(irfunc->frameType);
   IF_LOG { Logger::cout() << "casting to: " << *irfunc->frameType << '\n'; }
-  LLValue *val = DtoBitCast(ctx, frameType);
-
+  LLValue *val = DtoBitCast(ctx, pframeType);
+  llvm::StructType *currFrame = irfunc->frameType;
   // Make the DWARF variable address relative to the context pointer (ctx);
   // register all ops (offsetting, dereferencing) required to get there in the
   // following list.
@@ -131,14 +149,10 @@ DValue *DtoNestedVariable(const Loc &loc, Type *astype, VarDeclaration *vd,
   LLSmallVector<int64_t, 4> dwarfAddrOps;
 #endif
 
-  const auto offsetToNthField = [&val, &dwarfAddrOps](unsigned fieldIndex,
+  const auto offsetToNthField = [&val, &dwarfAddrOps, &currFrame](unsigned fieldIndex,
                                                       const char *name = "") {
-    gIR->DBuilder.OpOffset(dwarfAddrOps, val, fieldIndex);
-    val = DtoGEP(val, 0, fieldIndex, name);
-  };
-  const auto dereference = [&val, &dwarfAddrOps](const char *name = "") {
-    gIR->DBuilder.OpDeref(dwarfAddrOps);
-    val = DtoAlignedLoad(val, name);
+    gIR->DBuilder.OpOffset(dwarfAddrOps, currFrame, fieldIndex);
+    val = DtoGEP(currFrame, val, 0, fieldIndex, name);
   };
 
   const auto vardepth = irLocal->nestedDepth;
@@ -160,7 +174,10 @@ DValue *DtoNestedVariable(const Loc &loc, Type *astype, VarDeclaration *vd,
     IF_LOG Logger::println("Lower depth");
     offsetToNthField(vardepth);
     IF_LOG Logger::cout() << "Frame index: " << *val << '\n';
-    dereference((std::string(".frame.") + vdparent->toChars()).c_str());
+    currFrame = getIrFunc(fd)->frameType;
+    gIR->DBuilder.OpDeref(dwarfAddrOps);
+    val = DtoAlignedLoad(currFrame->getPointerTo(), val,
+                         (std::string(".frame.") + vdparent->toChars()).c_str());
     IF_LOG Logger::cout() << "Frame: " << *val << '\n';
   }
 
@@ -173,7 +190,7 @@ DValue *DtoNestedVariable(const Loc &loc, Type *astype, VarDeclaration *vd,
     // Handled appropriately by makeVarDValue() and EmitLocalVariable(), pass
     // storage of pointer (reference lvalue).
   } else if (byref || captureByRef(vd)) {
-    val = DtoAlignedLoad(val);
+    val = DtoAlignedLoad(irLocal->value->getType(), val);
     // ref/out variables get a reference-debuginfo-type in EmitLocalVariable()
     // => don't dereference, use reference lvalue as address
     if (!vd->isReference())
@@ -211,8 +228,9 @@ void DtoResolveNestedContext(const Loc &loc, AggregateDeclaration *decl,
     DtoResolveDsymbol(decl);
 
     unsigned idx = getVthisIdx(decl);
-    LLValue *gep = DtoGEP(value, 0, idx, ".vthis");
-    DtoStore(DtoBitCast(nest, gep->getType()->getContainedType(0)), gep);
+    llvm::StructType *st = getIrAggr(decl)->getLLStructType();
+    LLValue *gep = DtoGEP(st, value, 0, idx, ".vthis");
+    DtoStore(DtoBitCast(nest, st->getElementType(idx)), gep);
   }
 }
 
@@ -262,13 +280,13 @@ LLValue *DtoNestedContext(const Loc &loc, Dsymbol *sym) {
   } else if (irFunc.thisArg) {
     // or just have a this argument
     AggregateDeclaration *ad = irFunc.decl->isMember2();
-    val = ad->isClassDeclaration() ? DtoLoad(irFunc.thisArg) : irFunc.thisArg;
+    val = loadThisPtr(ad, irFunc);
     if (!ad->vthis) {
       // This is just a plain 'outer' reference of a class nested in a
       // function (but without any variables in the nested context).
       return val;
     }
-    val = DtoLoad(DtoGEP(val, 0, getVthisIdx(ad), ".vthis"));
+    val = indexVThis(ad, val);
   } else {
     if (sym->isFuncDeclaration()) {
       // If we are here, the function actually needs its nested context
@@ -291,7 +309,23 @@ LLValue *DtoNestedContext(const Loc &loc, Dsymbol *sym) {
     IF_LOG Logger::println("Current function is %s", ctxfd->toChars());
     if (fromParent) {
       ctxfd = getParentFunc(ctxfd);
-      assert(ctxfd && "Context from outer function, but no outer function?");
+
+      /*
+       The following can happen with
+       class Outer {
+           class Inner {
+               auto opSlice() {
+                   struct Range {
+                       void foo() {}
+                   }
+                   return Range();
+               }
+           }
+       }
+       see https://github.com/ldc-developers/ldc/issues/4130
+       */
+      if (!ctxfd)
+        ctxfd = irFunc.decl;
     }
     IF_LOG Logger::println("Context is from %s", ctxfd->toChars());
 
@@ -310,10 +344,10 @@ LLValue *DtoNestedContext(const Loc &loc, Dsymbol *sym) {
       IF_LOG Logger::println(
           "Calling sibling function or directly nested function");
     } else {
-      val = DtoBitCast(val,
-                       LLPointerType::getUnqual(getIrFunc(ctxfd)->frameType));
-      val = DtoGEP(val, 0, neededDepth);
-      val = DtoAlignedLoad(
+      llvm::StructType *type = getIrFunc(ctxfd)->frameType;
+      val = DtoBitCast(val, LLPointerType::getUnqual(type));
+      val = DtoGEP(type, val, 0, neededDepth);
+      val = DtoAlignedLoad(type->getElementType(neededDepth),
           val, (std::string(".frame.") + frameToPass->toChars()).c_str());
     }
   }
@@ -381,62 +415,66 @@ static void DtoCreateNestedContextType(FuncDeclaration *fd) {
   IF_LOG Logger::cout() << "Function " << fd->toChars() << " has depth "
                         << depth << '\n';
 
-  AggrTypeBuilder builder(false);
+  AggrTypeBuilder builder;
 
   if (depth != 0) {
     assert(innerFrameType);
-    unsigned ptrSize = gDataLayout->getPointerSize();
     // Add frame pointer types for all but last frame
     for (unsigned i = 0; i < (depth - 1); ++i) {
-      builder.addType(innerFrameType->getElementType(i), ptrSize);
+      builder.addType(innerFrameType->getElementType(i), target.ptrsize);
     }
     // Add frame pointer type for last frame
-    builder.addType(LLPointerType::getUnqual(innerFrameType), ptrSize);
+    builder.addType(LLPointerType::getUnqual(innerFrameType), target.ptrsize);
   }
 
   // Add the direct nested variables of this function, and update their
   // indices to match.
   // TODO: optimize ordering for minimal space usage?
+  unsigned maxAlignment = 1;
   for (auto vd : fd->closureVars) {
-    unsigned alignment = DtoAlignment(vd);
-    if (alignment > 1) {
-      builder.alignCurrentOffset(alignment);
-    }
-
     const bool isParam = vd->isParameter();
 
-    IrLocal &irLocal =
-        *(isParam ? getIrParameter(vd, true) : getIrLocal(vd, true));
-    irLocal.nestedIndex = builder.currentFieldIndex();
-    irLocal.nestedDepth = depth;
-
     LLType *t = nullptr;
+    unsigned alignment = 0;
     if (captureByRef(vd)) {
       t = DtoType(vd->type->pointerTo());
+      alignment = target.ptrsize;
     } else if (isParam && (vd->storage_class & STClazy)) {
       // the type is a delegate (LL struct)
       auto tf = TypeFunction::create(nullptr, vd->type, VARARGnone, LINK::d);
       auto td = TypeDelegate::create(tf);
       t = DtoType(td);
+      alignment = target.ptrsize;
     } else {
       t = DtoMemType(vd->type);
+      alignment = DtoAlignment(vd);
     }
 
+    if (alignment > 1) {
+      builder.alignCurrentOffset(alignment);
+      maxAlignment = std::max(maxAlignment, alignment);
+    }
+
+    IrLocal &irLocal =
+        *(isParam ? getIrParameter(vd, true) : getIrLocal(vd, true));
+    irLocal.nestedIndex = builder.currentFieldIndex();
+    irLocal.nestedDepth = depth;
+
     builder.addType(t, getTypeAllocSize(t));
 
     IF_LOG Logger::cout() << "Nested var '" << vd->toChars() << "' of type "
                           << *t << "\n";
   }
 
-  LLStructType *frameType =
-      LLStructType::create(gIR->context(), builder.defaultTypes(),
-                           std::string("nest.") + fd->toChars());
+  LLStructType *frameType = LLStructType::create(
+      gIR->context(), builder.defaultTypes(),
+      std::string("nest.") + fd->toChars(), builder.isPacked());
 
   IF_LOG Logger::cout() << "frameType = " << *frameType << '\n';
 
   // Store type in IrFunction
   irFunc.frameType = frameType;
-  irFunc.frameTypeAlignment = builder.overallAlignment();
+  irFunc.frameTypeAlignment = maxAlignment;
 }
 
 void DtoCreateNestedContext(FuncGenState &funcGen) {
@@ -451,17 +489,33 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
     auto &irFunc = funcGen.irFunc;
     unsigned depth = irFunc.depth;
     LLStructType *frameType = irFunc.frameType;
+    const unsigned frameAlignment =
+        std::max(getABITypeAlign(frameType), irFunc.frameTypeAlignment);
+
     // Create frame for current function and append to frames list
     LLValue *frame = nullptr;
     bool needsClosure = fd->needsClosure();
     IF_LOG Logger::println("Needs closure (GC) flag: %d", (int)needsClosure);
     if (needsClosure) {
-      // FIXME: alignment ?
-      frame = DtoGcMalloc(fd->loc, frameType, ".frame");
+      LLFunction *fn =
+          getRuntimeFunction(fd->loc, gIR->module, "_d_allocmemory");
+      auto size = getTypeAllocSize(frameType);
+      if (frameAlignment > 16) // GC guarantees an alignment of 16
+        size += frameAlignment - 16;
+      LLValue *mem =
+          gIR->CreateCallOrInvoke(fn, DtoConstSize_t(size), ".gc_frame");
+      if (frameAlignment <= 16) {
+        frame = DtoBitCast(mem, frameType->getPointerTo(), ".frame");
+      } else {
+        const uint64_t mask = frameAlignment - 1;
+        mem = gIR->ir->CreatePtrToInt(mem, DtoSize_t());
+        mem = gIR->ir->CreateAdd(mem, DtoConstSize_t(mask));
+        mem = gIR->ir->CreateAnd(mem, DtoConstSize_t(~mask));
+        frame =
+            gIR->ir->CreateIntToPtr(mem, frameType->getPointerTo(), ".frame");
+      }
     } else {
-      unsigned alignment =
-          std::max(getABITypeAlign(frameType), irFunc.frameTypeAlignment);
-      frame = DtoRawAlloca(frameType, alignment, ".frame");
+      frame = DtoRawAlloca(frameType, frameAlignment, ".frame");
     }
 
     // copy parent frames into beginning
@@ -472,10 +526,9 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
         AggregateDeclaration *ad = fd->isMember2();
         assert(ad);
         assert(ad->vthis);
-        LLValue *thisptr =
-            ad->isClassDeclaration() ? DtoLoad(irFunc.thisArg) : irFunc.thisArg;
+        LLValue *thisptr = loadThisPtr(ad, irFunc);
         IF_LOG Logger::println("Indexing to 'this'");
-        src = DtoLoad(DtoGEP(thisptr, 0, getVthisIdx(ad), ".vthis"));
+        src = indexVThis(ad, thisptr);
       }
       if (depth > 1) {
         src = DtoBitCast(src, getVoidPtrType());
@@ -486,7 +539,7 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
       // Copy nestArg into framelist; the outer frame is not in the list of
       // pointers
       src = DtoBitCast(src, frameType->getContainedType(depth - 1));
-      LLValue *gep = DtoGEP(frame, 0, depth - 1);
+      LLValue *gep = DtoGEP(frameType, frame, 0, depth - 1);
       DtoAlignedStore(src, gep);
     }
 
@@ -501,7 +554,7 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
       }
 
       IrLocal *irLocal = getIrLocal(vd);
-      LLValue *gep = DtoGEP(frame, 0, irLocal->nestedIndex, vd->toChars());
+      LLValue *gep = DtoGEP(frameType, frame, 0, irLocal->nestedIndex, vd->toChars());
       if (vd->isParameter()) {
         IF_LOG Logger::println("nested param: %s", vd->toChars());
         LOG_SCOPE
@@ -519,7 +572,7 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
           // The parameter value is an alloca'd stack slot.
           // Copy to the nesting frame and leave the alloca for
           // the optimizers to clean up.
-          DtoMemCpy(gep, parm->value);
+          DtoMemCpy(frameType->getContainedType(irLocal->nestedIndex), gep, parm->value);
           gep->takeName(parm->value);
           parm->value = gep; // update variable lvalue
         }
