@@ -18,7 +18,7 @@
 #include "dmd/target.h"
 #include "dmd/tokens.h"
 #include "driver/cl_options_instrumentation.h"
-#include "gen/abi.h"
+#include "gen/abi/abi.h"
 #include "gen/attributes.h"
 #include "gen/functions.h"
 #include "gen/irstate.h"
@@ -32,6 +32,9 @@
 #include "ir/irtypefunction.h"
 #include "llvm/Bitcode/BitcodeWriter.h"
 #include "llvm/IR/Attributes.h"
+#if LDC_LLVM_VER >= 1600
+#include "llvm/Support/ModRef.h"
+#endif
 #include "llvm/IR/Module.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/MemoryBuffer.h"
@@ -60,6 +63,7 @@ static void checkForImplicitGCCall(const Loc &loc, const char *name) {
         "_aaDelX",
         "_aaGetY",
         "_aaKeys",
+        "_aaNew",
         "_aaRehash",
         "_aaValues",
         "_d_allocmemory",
@@ -69,8 +73,6 @@ static void checkForImplicitGCCall(const Loc &loc, const char *name) {
         "_d_arrayappendcTX",
         "_d_arrayappendcd",
         "_d_arrayappendwd",
-        "_d_arraycatT",
-        "_d_arraycatnTX",
         "_d_arraysetlengthT",
         "_d_arraysetlengthiT",
         "_d_assocarrayliteralTX",
@@ -87,8 +89,7 @@ static void checkForImplicitGCCall(const Loc &loc, const char *name) {
         "_d_newarrayU",
         "_d_newclass",
         "_d_allocclass",
-        "_d_newitemT",
-        "_d_newitemiT",
+        // TODO: _d_newitemT instantiations
     };
 
     if (binary_search(&GCNAMES[0],
@@ -272,7 +273,11 @@ struct LazyFunctionDeclarer {
       // FIXME: Move to better place (abi-x86-64.cpp?)
       // NOTE: There are several occurances if this line.
       if (global.params.targetTriple->getArch() == llvm::Triple::x86_64) {
+#if LDC_LLVM_VER >= 1500
+        fn->setUWTableKind(llvm::UWTableKind::Default);
+#else
         fn->addFnAttr(LLAttribute::UWTable);
+#endif
       }
 
       fn->setCallingConv(gABI->callingConv(dty, false));
@@ -356,6 +361,8 @@ llvm::Function *getRuntimeFunction(const Loc &loc, llvm::Module &target,
 //                             const char *funcname);
 // uClibc:  void __assert(const char *assertion, const char *filename, int linenumber,
 //                        const char *function);
+// newlib:  void __assert_func(const char *file, int line, const char *func,
+//                             const char *failedexpr)
 // else:    void __assert(const char *msg, const char *file, unsigned line)
 
 static const char *getCAssertFunctionName() {
@@ -368,6 +375,8 @@ static const char *getCAssertFunctionName() {
     return "__assert_c99";
   } else if (triple.isMusl()) {
     return "__assert_fail";
+  } else if (global.params.isNewlibEnvironment) {
+    return "__assert_func";
   }
   return "__assert";
 }
@@ -384,6 +393,9 @@ static std::vector<PotentiallyLazyType> getCAssertFunctionParamTypes() {
   if (triple.getEnvironment() == llvm::Triple::Android) {
     return {voidPtr, uint, voidPtr};
   }
+  if (global.params.isNewlibEnvironment) {
+    return {voidPtr, uint, voidPtr, voidPtr};
+  }
   return {voidPtr, voidPtr, uint};
 }
 
@@ -464,7 +476,8 @@ static Type *rt_dg2() {
 
 static void buildRuntimeModule() {
   Logger::println("building runtime module");
-  M = new llvm::Module("ldc internal runtime", gIR->context());
+  auto &context = gIR->context();
+  M = new llvm::Module("ldc internal runtime", context);
 
   Type *voidTy = Type::tvoid;
   Type *boolTy = Type::tbool;
@@ -493,8 +506,14 @@ static void buildRuntimeModule() {
   AttrSet NoAttrs,
       Attr_NoUnwind(NoAttrs, LLAttributeList::FunctionIndex,
                     llvm::Attribute::NoUnwind),
+#if LDC_LLVM_VER >= 1600
+      Attr_ReadOnly(llvm::AttributeList().addFnAttribute(
+          context, llvm::Attribute::getWithMemoryEffects(
+                       context, llvm::MemoryEffects::readOnly()))),
+#else
       Attr_ReadOnly(NoAttrs, LLAttributeList::FunctionIndex,
                     llvm::Attribute::ReadOnly),
+#endif
       Attr_Cold(NoAttrs, LLAttributeList::FunctionIndex, llvm::Attribute::Cold),
       Attr_Cold_NoReturn(Attr_Cold, LLAttributeList::FunctionIndex,
                          llvm::Attribute::NoReturn),
@@ -584,33 +603,11 @@ static void buildRuntimeModule() {
   createFwdDecl(LINK::c, voidArrayTy, {"_d_newarraymTX", "_d_newarraymiTX"},
                 {typeInfoTy, sizeTy->arrayOf()}, {STCconst, 0});
 
-  // void[] _d_arraysetlengthT (const TypeInfo ti, size_t newlength, void[]* p)
-  // void[] _d_arraysetlengthiT(const TypeInfo ti, size_t newlength, void[]* p)
-  createFwdDecl(LINK::c, voidArrayTy,
-                {"_d_arraysetlengthT", "_d_arraysetlengthiT"},
-                {typeInfoTy, sizeTy, voidArrayPtrTy}, {STCconst, 0, 0});
-
-  // byte[] _d_arrayappendcTX(const TypeInfo ti, ref byte[] px, size_t n)
-  createFwdDecl(LINK::c, voidArrayTy, {"_d_arrayappendcTX"},
-                {typeInfoTy, voidArrayTy, sizeTy}, {STCconst, STCref, 0});
-
-  // void[] _d_arrayappendT(const TypeInfo ti, ref byte[] x, byte[] y)
-  createFwdDecl(LINK::c, voidArrayTy, {"_d_arrayappendT"},
-                {typeInfoTy, voidArrayTy, voidArrayTy}, {STCconst, STCref, 0});
-
   // void[] _d_arrayappendcd(ref byte[] x, dchar c)
   // void[] _d_arrayappendwd(ref byte[] x, dchar c)
   createFwdDecl(LINK::c, voidArrayTy, {"_d_arrayappendcd", "_d_arrayappendwd"},
                 {voidArrayTy, dcharTy}, {STCref, 0});
 
-  // byte[] _d_arraycatT(const TypeInfo ti, byte[] x, byte[] y)
-  createFwdDecl(LINK::c, voidArrayTy, {"_d_arraycatT"},
-                {typeInfoTy, voidArrayTy, voidArrayTy}, {STCconst, 0, 0});
-
-  // void[] _d_arraycatnTX(const TypeInfo ti, byte[][] arrs)
-  createFwdDecl(LINK::c, voidArrayTy, {"_d_arraycatnTX"},
-                {typeInfoTy, voidArrayTy->arrayOf()}, {STCconst, 0});
-
   // Object _d_newclass(const ClassInfo ci)
   // Object _d_allocclass(const ClassInfo ci)
   createFwdDecl(LINK::c, objectTy, {"_d_newclass", "_d_allocclass"},
@@ -620,11 +617,6 @@ static void buildRuntimeModule() {
   createFwdDecl(LINK::c, throwableTy, {"_d_newThrowable"}, {classInfoTy},
                 {STCconst});
 
-  // void* _d_newitemT (TypeInfo ti)
-  // void* _d_newitemiT(TypeInfo ti)
-  createFwdDecl(LINK::c, voidPtrTy, {"_d_newitemT", "_d_newitemiT"},
-                {typeInfoTy}, {0});
-
   // void _d_delarray_t(void[]* p, const TypeInfo_Struct ti)
   createFwdDecl(LINK::c, voidTy, {"_d_delarray_t"},
                 {voidArrayPtrTy, structTypeInfoTy}, {0, STCconst});
@@ -696,13 +688,8 @@ static void buildRuntimeModule() {
   createFwdDecl(LINK::c, voidArrayTy, {"_d_arrayassign_l", "_d_arrayassign_r"},
                 {typeInfoTy, voidArrayTy, voidArrayTy, voidPtrTy});
 
-  // void[] _d_arrayctor(TypeInfo ti, void[] from, void[] to)
-  createFwdDecl(LINK::c, voidArrayTy, {"_d_arrayctor"},
-                {typeInfoTy, voidArrayTy, voidArrayTy});
-
   // void* _d_arraysetassign(void* p, void* value, int count, TypeInfo ti)
-  // void* _d_arraysetctor(void* p, void* value, int count, TypeInfo ti)
-  createFwdDecl(LINK::c, voidPtrTy, {"_d_arraysetassign", "_d_arraysetctor"},
+  createFwdDecl(LINK::c, voidPtrTy, {"_d_arraysetassign"},
                 {voidPtrTy, voidPtrTy, intTy, typeInfoTy});
 
   //////////////////////////////////////////////////////////////////////////////
@@ -755,6 +742,9 @@ static void buildRuntimeModule() {
   createFwdDecl(LINK::c, aaTy, {"_d_assocarrayliteralTX"},
                 {aaTypeInfoTy, voidArrayTy, voidArrayTy}, {STCconst, 0, 0});
 
+  // AA _aaNew(const TypeInfo_AssociativeArray ti)
+  createFwdDecl(LINK::c, aaTy, {"_aaNew"}, {aaTypeInfoTy}, {STCconst});
+
   //////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////
