@@ -18,7 +18,7 @@
 #include "dmd/module.h"
 #include "dmd/mtype.h"
 #include "dmd/root/port.h"
-#include "gen/abi.h"
+#include "gen/abi/abi.h"
 #include "gen/arrays.h"
 #include "gen/classes.h"
 #include "gen/coverage.h"
@@ -258,7 +258,7 @@ public:
         if (returnValue->getType() != funcType->getReturnType() &&
             DtoIsInMemoryOnly(rt) && isaPointer(returnValue)) {
           Logger::println("Loading value for return");
-          returnValue = DtoLoad(returnValue);
+          returnValue = DtoLoad(funcType->getReturnType(), returnValue);
         }
 
         // can happen for classes
@@ -309,12 +309,13 @@ public:
         // Hack: the frontend generates 'return 0;' as last statement of
         // 'void main()'. But the debug location is missing. Use the end
         // of function as debug location.
-        if (isAnyMainFunction(fd) && !stmt->loc.linnum) {
+        if (isAnyMainFunction(fd) && !stmt->loc.linnum()) {
           irs->DBuilder.EmitStopPoint(fd->endloc);
         }
 
-        irs->ir->CreateRet(useRetValSlot ? DtoLoad(funcGen.retValSlot)
-                                         : returnValue);
+        irs->ir->CreateRet(
+            useRetValSlot ? DtoLoad(funcType->getReturnType(), funcGen.retValSlot)
+                          : returnValue);
       } else {
         irs->ir->CreateRetVoid();
       }
@@ -363,7 +364,7 @@ public:
     auto arg2 = (*ce->arguments)[1]->toInteger();
     auto dct = irs->dcomputetarget;
     if (!dct) {
-      return arg1 == DComputeTarget::Host;
+      return arg1 == DComputeTarget::ID::Host;
     } else {
       return arg1 == dct->target &&
              (!arg2 || arg2 == static_cast<dinteger_t>(dct->tversion));
@@ -396,6 +397,9 @@ public:
     // start a dwarf lexical block
     irs->DBuilder.EmitBlockStart(stmt->loc);
     emitCoverageLinecountInc(stmt->loc);
+    // Open a new scope for the optional condition variable (`if (auto i = ...)`)
+    irs->funcGen().localVariableLifetimeAnnotator.pushScope();
+
 
     // This is a (dirty) hack to get codegen time conditional
     // compilation, on account of the fact that we are trying
@@ -481,6 +485,9 @@ public:
 
     // rewrite the scope
     irs->ir->SetInsertPoint(endbb);
+    // Close the scope for the optional condition variable. This is suboptimal,
+    // because the condition variable is not in scope in the else block.
+    irs->funcGen().localVariableLifetimeAnnotator.popScope();
   }
 
   //////////////////////////////////////////////////////////////////////////
@@ -493,9 +500,11 @@ public:
     PGO.setCurrentStmt(stmt);
 
     if (stmt->statement) {
+      irs->funcGen().localVariableLifetimeAnnotator.pushScope();
       irs->DBuilder.EmitBlockStart(stmt->statement->loc);
       stmt->statement->accept(this);
       irs->DBuilder.EmitBlockEnd();
+      irs->funcGen().localVariableLifetimeAnnotator.popScope();
     }
   }
 
@@ -635,6 +644,7 @@ public:
 
     // start new dwarf lexical block
     irs->DBuilder.EmitBlockStart(stmt->loc);
+    irs->funcGen().localVariableLifetimeAnnotator.pushScope();
 
     // create for blocks
     llvm::BasicBlock *forbb = irs->insertBB("forcond");
@@ -716,6 +726,7 @@ public:
     irs->ir->SetInsertPoint(endbb);
 
     // end the dwarf lexical block
+    irs->funcGen().localVariableLifetimeAnnotator.popScope();
     irs->DBuilder.EmitBlockEnd();
   }
 
@@ -1345,7 +1356,7 @@ public:
     irs->ir->SetInsertPoint(condbb);
 
     LLValue *done = nullptr;
-    LLValue *load = DtoLoad(keyvar);
+    LLValue *load = DtoLoad(keytype, keyvar);
     if (stmt->op == TOK::foreach_) {
       done = irs->ir->CreateICmpULT(load, niters);
     } else if (stmt->op == TOK::foreach_reverse_) {
@@ -1365,8 +1376,8 @@ public:
     PGO.emitCounterIncrement(stmt);
 
     // get value for this iteration
-    LLValue *loadedKey = DtoLoad(keyvar);
-    LLValue *gep = DtoGEP1(val, loadedKey);
+    LLValue *loadedKey = DtoLoad(keytype, keyvar);
+    LLValue *gep = DtoGEP1(DtoMemType(aggrval->type->nextOf()), val, loadedKey);
 
     if (!stmt->value->isRef() && !stmt->value->isOut()) {
       // Copy value to local variable, and use it as the value variable.
@@ -1393,7 +1404,7 @@ public:
     // next
     irs->ir->SetInsertPoint(nextbb);
     if (stmt->op == TOK::foreach_) {
-      LLValue *load = DtoLoad(keyvar);
+      LLValue *load = DtoLoad(keytype, keyvar);
       load = irs->ir->CreateAdd(load, LLConstantInt::get(keytype, 1, false));
       DtoStore(load, keyvar);
     }
@@ -1427,8 +1438,8 @@ public:
 
     // handle key
     assert(stmt->key->type->isintegral());
-    LLValue *keyval = DtoRawVarDeclaration(stmt->key);
-
+    LLValue *keyval  = DtoRawVarDeclaration(stmt->key);
+    LLType  *keytype = DtoType(stmt->key->type);
     // store initial value in key
     if (stmt->op == TOK::foreach_) {
       DtoStore(lower, keyval);
@@ -1449,7 +1460,7 @@ public:
     irs->ir->SetInsertPoint(condbb);
 
     // first we test that lwr < upr
-    lower = DtoLoad(keyval);
+    lower = DtoLoad(keytype, keyval);
     assert(lower->getType() == upper->getType());
     llvm::ICmpInst::Predicate cmpop;
     if (isLLVMUnsigned(stmt->key->type)) {
@@ -1475,7 +1486,7 @@ public:
 
     // reverse foreach decrements here
     if (stmt->op == TOK::foreach_reverse_) {
-      LLValue *v = DtoLoad(keyval);
+      LLValue *v = DtoLoad(keytype, keyval);
       LLValue *one = LLConstantInt::get(v->getType(), 1, false);
       v = irs->ir->CreateSub(v, one);
       DtoStore(v, keyval);
@@ -1498,7 +1509,7 @@ public:
 
     // forward foreach increments here
     if (stmt->op == TOK::foreach_) {
-      LLValue *v = DtoLoad(keyval);
+      LLValue *v = DtoLoad(keytype, keyval);
       LLValue *one = LLConstantInt::get(v->getType(), 1, false);
       v = irs->ir->CreateAdd(v, one);
       DtoStore(v, keyval);
