@@ -165,13 +165,14 @@ public:
     Type *const t = e->type->toBasetype();
 
     if (auto ts = t->isTypeSArray()) {
-      bool zeroTerm = ts->dim->toInteger() == e->numberOfCodeUnits() + 1;
-      result = buildStringLiteralConstant(e, zeroTerm);
+      const auto arrayLength = ts->dim->toInteger();
+      assert(arrayLength >= e->numberOfCodeUnits());
+      result = buildStringLiteralConstant(e, arrayLength);
       return;
     }
 
     llvm::GlobalVariable *gvar = p->getCachedStringLiteral(e);
-    LLConstant *arrptr = DtoGEP(gvar, 0u, 0u);
+    LLConstant *arrptr = DtoGEP(gvar->getValueType(), gvar, 0u, 0u);
 
     if (t->ty == TY::Tpointer) {
       result = DtoBitCast(arrptr, DtoType(t));
@@ -186,18 +187,45 @@ public:
 
   //////////////////////////////////////////////////////////////////////////////
 
+  // very similar to emitPointerOffset() in binops.cpp
+  LLConstant *tryEmitPointerOffset(BinExp *e, bool negateOffset) {
+    Type *t1b = e->e1->type->toBasetype();
+    if (t1b->ty != TY::Tpointer || !e->e2->type->isintegral())
+      return nullptr;
+
+    Type *const pointeeType = t1b->nextOf();
+
+    LLConstant *llBase = toConstElem(e->e1, p);
+    const dinteger_t byteOffset = e->e2->toInteger();
+
+    LLConstant *llResult = nullptr;
+    const auto pointeeSize = pointeeType->size(e->loc);
+    if (pointeeSize && byteOffset % pointeeSize == 0) { // can do a nice GEP
+      LLConstant *llOffset = DtoConstSize_t(byteOffset / pointeeSize);
+      if (negateOffset)
+        llOffset = llvm::ConstantExpr::getNeg(llOffset);
+      llResult = llvm::ConstantExpr::getGetElementPtr(DtoMemType(pointeeType),
+                                                      llBase, llOffset);
+    } else { // need to cast base to i8*
+      llBase = DtoBitCast(llBase, getVoidPtrType());
+      LLConstant *llOffset = DtoConstSize_t(byteOffset);
+      if (negateOffset)
+        llOffset = llvm::ConstantExpr::getNeg(llOffset);
+      llResult =
+          llvm::ConstantExpr::getGetElementPtr(getI8Type(), llBase, llOffset);
+    }
+
+    return DtoBitCast(llResult, DtoType(e->type));
+  }
+
   void visit(AddExp *e) override {
     IF_LOG Logger::print("AddExp::toConstElem: %s @ %s\n", e->toChars(),
                          e->type->toChars());
     LOG_SCOPE;
 
     // add to pointer
-    Type *t1b = e->e1->type->toBasetype();
-    if (t1b->ty == TY::Tpointer && e->e2->type->isintegral()) {
-      llvm::Constant *ptr = toConstElem(e->e1, p);
-      dinteger_t idx = undoStrideMul(e->loc, t1b, e->e2->toInteger());
-      result = llvm::ConstantExpr::getGetElementPtr(getPointeeType(ptr), ptr,
-                                                    DtoConstSize_t(idx));
+    if (auto r = tryEmitPointerOffset(e, false)) {
+      result = r;
       return;
     }
 
@@ -209,14 +237,9 @@ public:
                          e->type->toChars());
     LOG_SCOPE;
 
-    Type *t1b = e->e1->type->toBasetype();
-    if (t1b->ty == TY::Tpointer && e->e2->type->isintegral()) {
-      llvm::Constant *ptr = toConstElem(e->e1, p);
-      dinteger_t idx = undoStrideMul(e->loc, t1b, e->e2->toInteger());
-
-      llvm::Constant *negIdx = llvm::ConstantExpr::getNeg(DtoConstSize_t(idx));
-      result = llvm::ConstantExpr::getGetElementPtr(getPointeeType(ptr), ptr,
-                                                    negIdx);
+    // subtract from pointer
+    if (auto r = tryEmitPointerOffset(e, true)) {
+      result = r;
       return;
     }
 
@@ -284,14 +307,15 @@ public:
           static_cast<VarExp *>(e->e1)->var->isVarDeclaration();
       assert(vd);
       DtoResolveVariable(vd);
+      IrGlobal *irg = getIrGlobal(vd);
       LLConstant *value =
-          isIrGlobalCreated(vd) ? isaConstant(getIrGlobal(vd)->value) : nullptr;
+          isIrGlobalCreated(vd) ? isaConstant(irg->value) : nullptr;
       if (!value) {
         goto Lerr;
       }
       Type *type = vd->type->toBasetype();
       if (type->ty == TY::Tarray || type->ty == TY::Tdelegate) {
-        value = DtoGEP(value, 0u, 1u);
+        value = DtoGEP(irg->getType(), value, 0u, 1u);
       }
       result = DtoBitCast(value, DtoType(tb));
     } else if (tb->ty == TY::Tclass && e->e1->type->ty == TY::Tclass &&
@@ -309,7 +333,7 @@ public:
         assert(i_index != ~0UL);
 
         // offset pointer
-        instance = DtoGEP(instance, 0, i_index);
+        instance = DtoGEP(DtoType(e->e1->type), instance, 0, i_index);
       }
       result = DtoBitCast(instance, DtoType(tb));
     } else {
@@ -340,7 +364,7 @@ public:
       result = base;
     } else {
       const unsigned elemSize =
-          gDataLayout->getTypeStoreSize(base->getType()->getContainedType(0));
+          gDataLayout->getTypeStoreSize(DtoType(e->var->type));
 
       IF_LOG Logger::println("adding offset: %llu (elem size: %u)",
                              static_cast<unsigned long long>(e->offset),
@@ -350,7 +374,7 @@ public:
       if (elemSize && e->offset % elemSize == 0) {
         // We can turn this into a "nice" GEP.
         result = llvm::ConstantExpr::getGetElementPtr(
-            getPointeeType(base), base, DtoConstSize_t(e->offset / elemSize));
+            DtoType(e->var->type), base, DtoConstSize_t(e->offset / elemSize));
       } else {
         // Offset isn't a multiple of base type size, just cast to i8* and
         // apply the byte offset.
@@ -400,7 +424,7 @@ public:
       LLConstant *val = isaConstant(getIrGlobal(vd)->value);
       val = DtoBitCast(val, DtoType(vd->type->pointerTo()));
       LLConstant *gep = llvm::ConstantExpr::getGetElementPtr(
-          getPointeeType(val), val, idxs, true);
+          DtoType(vd->type), val, idxs, true);
 
       // bitcast to requested type
       assert(e->type->toBasetype()->ty == TY::Tpointer);
@@ -419,7 +443,7 @@ public:
       auto globalVar = new llvm::GlobalVariable(
           p->module, DtoType(se->type), false,
           llvm::GlobalValue::InternalLinkage, nullptr, ".structliteral");
-      globalVar->setAlignment(LLMaybeAlign(DtoAlignment(se->type)));
+      globalVar->setAlignment(llvm::MaybeAlign(DtoAlignment(se->type)));
 
       p->setStructLiteralConstant(se, globalVar);
       llvm::Constant *constValue = toConstElem(se, p);
@@ -527,7 +551,7 @@ public:
 
     // build a constant dynamic array reference with the .ptr field pointing
     // into store
-    LLConstant *globalstorePtr = DtoGEP(store, 0u, 0u);
+    LLConstant *globalstorePtr = DtoGEP(arrtype, store, 0u, 0u);
     result = DtoConstSlice(DtoConstSize_t(e->elements->length), globalstorePtr);
   }
 
@@ -634,13 +658,13 @@ public:
       if (InterfaceDeclaration *it = targetClass->isInterfaceDeclaration()) {
         assert(it->isBaseOf(origClass, NULL));
 
+        IrTypeClass *itc = getIrType(origClass->type)->isClass();
         // find interface impl
-        size_t i_index =
-            getIrType(origClass->type)->isClass()->getInterfaceIndex(it);
+        size_t i_index = itc->getInterfaceIndex(it);
         assert(i_index != ~0UL);
 
         // offset pointer
-        result = DtoGEP(result, 0, i_index);
+        result = DtoGEP(itc->getMemoryLLType(), result, 0, i_index);
       }
     }
 
@@ -684,10 +708,8 @@ public:
       //        constructed.
 #if LDC_LLVM_VER >= 1200
       const auto elementCount = llvm::ElementCount::getFixed(elemCount);
-#elif LDC_LLVM_VER >= 1100
-      const auto elementCount = llvm::ElementCount(elemCount, false);
 #else
-      const auto elementCount = elemCount;
+      const auto elementCount = llvm::ElementCount(elemCount, false);
 #endif
       result = llvm::ConstantVector::getSplat(
           elementCount, toConstElem(e->e1->optimize(WANTvalue), p));
@@ -712,6 +734,12 @@ public:
 
   //////////////////////////////////////////////////////////////////////////////
 
+  void visit(AssocArrayLiteralExp *e) override {
+    e->error("static initializations of associative arrays is not allowed.");
+    errorSupplemental(e->loc, "associative arrays must be initialized at runtime: https://dlang.org/spec/hash-map.html#runtime_initialization");
+    fatalError();
+  }
+
   void visit(Expression *e) override {
     e->error("expression `%s` is not a constant", e->toChars());
     fatalError();
