@@ -18,7 +18,7 @@
 #include "dmd/module.h"
 #include "dmd/target.h"
 #include "driver/cl_options.h"
-#include "gen/abi.h"
+#include "gen/abi/abi.h"
 #include "gen/arrays.h"
 #include "gen/classes.h"
 #include "gen/complex.h"
@@ -178,7 +178,7 @@ LLType *DtoType(Type *t) {
       // This is an enum forward reference that is only legal when referenced
       // through an indirection (e.g. "enum E; void foo(E* p);"). For lack of a
       // better choice, make the outer indirection a void pointer.
-      return getVoidPtrType()->getContainedType(0);
+      return getI8Type();
     }
     return DtoType(bt);
   }
@@ -227,28 +227,39 @@ LLValue *DtoDelegateEquals(EXP op, LLValue *lhs, LLValue *rhs) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
+namespace {
+LLGlobalValue::LinkageTypes DtoLinkageOnly(Dsymbol *sym) {
+  if (hasWeakUDA(sym))
+    return LLGlobalValue::WeakAnyLinkage;
+
+  // static in ImportC translates to internal linkage
+  if (auto decl = sym->isDeclaration())
+    if ((decl->storage_class & STCstatic) && decl->isCsymbol())
+      return LLGlobalValue::InternalLinkage;
+
+  /* Function (incl. delegate) literals are emitted into each referencing
+   * compilation unit, so use internal linkage for all lambdas and all global
+   * variables they define.
+   * This makes sure these symbols don't accidentally collide when linking
+   * object files compiled by different compiler invocations (lambda mangles
+   * aren't stable - see https://issues.dlang.org/show_bug.cgi?id=23722).
+   */
+  auto potentialLambda = sym;
+  if (auto vd = sym->isVarDeclaration())
+    if (vd->isDataseg())
+      potentialLambda = vd->toParent2();
+  if (potentialLambda->isFuncLiteralDeclaration())
+    return LLGlobalValue::InternalLinkage;
+
+  if (sym->isInstantiated())
+    return templateLinkage;
+
+  return LLGlobalValue::ExternalLinkage;
+}
+}
+
 LinkageWithCOMDAT DtoLinkage(Dsymbol *sym) {
-  LLGlobalValue::LinkageTypes linkage = LLGlobalValue::ExternalLinkage;
-  if (hasWeakUDA(sym)) {
-    linkage = LLGlobalValue::WeakAnyLinkage;
-  } else {
-    // Function (incl. delegate) literals are emitted into each referencing
-    // compilation unit, so use linkonce_odr for all lambdas and all global
-    // variables they define.
-    auto potentialLambda = sym;
-    if (auto vd = sym->isVarDeclaration()) {
-      if (vd->isDataseg())
-        potentialLambda = vd->toParent2();
-    }
-
-    if (potentialLambda->isFuncLiteralDeclaration()) {
-      linkage = LLGlobalValue::LinkOnceODRLinkage;
-    } else if (sym->isInstantiated()) {
-      linkage = templateLinkage;
-    }
-  }
-
-  return {linkage, needsCOMDAT()};
+  return {DtoLinkageOnly(sym), needsCOMDAT()};
 }
 
 bool needsCOMDAT() {
@@ -309,8 +320,9 @@ void setVisibility(Dsymbol *sym, llvm::GlobalObject *obj) {
   } else {
     if (sym->isExport()) {
       obj->setVisibility(LLGlobalValue::DefaultVisibility); // overrides @hidden
-    } else if (!hasHiddenUDA) {
+    } else if (!obj->hasLocalLinkage() && !hasHiddenUDA) {
       // Hide with -fvisibility=hidden, or linkonce_odr etc.
+      // Note that symbols with local linkage cannot be hidden (LLVM assertion).
       // The Apple linker warns about hidden linkonce_odr symbols from object
       // files compiled with -linkonce-templates being folded with *public*
       // weak_odr symbols from non-linkonce-templates code (e.g., Phobos), so
@@ -345,50 +357,51 @@ LLIntegerType *DtoSize_t() {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-LLType *getPointeeType(LLValue *pointer) {
-  return pointer->getType()->getPointerElementType();
-}
-
-////////////////////////////////////////////////////////////////////////////////
-
 namespace {
-llvm::GetElementPtrInst *DtoGEP(LLValue *ptr, llvm::ArrayRef<LLValue *> indices,
+llvm::GetElementPtrInst *DtoGEP(LLType *pointeeTy, LLValue *ptr,
+                                llvm::ArrayRef<LLValue *> indices,
                                 const char *name, llvm::BasicBlock *bb) {
-  auto gep = llvm::GetElementPtrInst::Create(getPointeeType(ptr), ptr, indices,
-                                             name, bb ? bb : gIR->scopebb());
+  auto gep = llvm::GetElementPtrInst::Create(pointeeTy, ptr, indices, name,
+                                             bb ? bb : gIR->scopebb());
   gep->setIsInBounds(true);
   return gep;
 }
 }
 
-LLValue *DtoGEP1(LLValue *ptr, LLValue *i0, const char *name,
+LLValue *DtoGEP1(LLType *pointeeTy, LLValue *ptr, LLValue *i0, const char *name,
                  llvm::BasicBlock *bb) {
-  return DtoGEP(ptr, i0, name, bb);
+  return DtoGEP(pointeeTy, ptr, i0, name, bb);
 }
 
-LLValue *DtoGEP(LLValue *ptr, LLValue *i0, LLValue *i1, const char *name,
-                llvm::BasicBlock *bb) {
+LLValue *DtoGEP(LLType *pointeeTy, LLValue *ptr, LLValue *i0, LLValue *i1,
+                const char *name, llvm::BasicBlock *bb) {
   LLValue *indices[] = {i0, i1};
-  return DtoGEP(ptr, indices, name, bb);
+  return DtoGEP(pointeeTy, ptr, indices, name, bb);
 }
 
-LLValue *DtoGEP1(LLValue *ptr, unsigned i0, const char *name,
+LLValue *DtoGEP1(LLType *pointeeTy, LLValue *ptr, unsigned i0, const char *name,
                  llvm::BasicBlock *bb) {
-  return DtoGEP(ptr, DtoConstUint(i0), name, bb);
+  return DtoGEP(pointeeTy, ptr, DtoConstUint(i0), name, bb);
 }
 
-LLValue *DtoGEP(LLValue *ptr, unsigned i0, unsigned i1, const char *name,
-                llvm::BasicBlock *bb) {
+LLValue *DtoGEP(LLType *pointeeTy, LLValue *ptr, unsigned i0, unsigned i1,
+                const char *name, llvm::BasicBlock *bb) {
   LLValue *indices[] = {DtoConstUint(i0), DtoConstUint(i1)};
-  return DtoGEP(ptr, indices, name, bb);
+  return DtoGEP(pointeeTy, ptr, indices, name, bb);
 }
 
-LLConstant *DtoGEP(LLConstant *ptr, unsigned i0, unsigned i1) {
+LLConstant *DtoGEP(LLType *pointeeTy, LLConstant *ptr, unsigned i0,
+                   unsigned i1) {
   LLValue *indices[] = {DtoConstUint(i0), DtoConstUint(i1)};
-  return llvm::ConstantExpr::getGetElementPtr(getPointeeType(ptr), ptr, indices,
+  return llvm::ConstantExpr::getGetElementPtr(pointeeTy, ptr, indices,
                                               /* InBounds = */ true);
 }
 
+LLValue *DtoGEP1i64(LLType *pointeeTy, LLValue *ptr, uint64_t i0, const char *name,
+                    llvm::BasicBlock *bb) {
+  return DtoGEP(pointeeTy, ptr, DtoConstUlong(i0), name, bb);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 void DtoMemSet(LLValue *dst, LLValue *val, LLValue *nbytes, unsigned align) {
@@ -396,18 +409,19 @@ void DtoMemSet(LLValue *dst, LLValue *val, LLValue *nbytes, unsigned align) {
 
   dst = DtoBitCast(dst, VoidPtrTy);
 
-  gIR->ir->CreateMemSet(dst, val, nbytes, LLMaybeAlign(align), false /*isVolatile*/);
+  gIR->ir->CreateMemSet(dst, val, nbytes, llvm::MaybeAlign(align),
+                        false /*isVolatile*/);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void DtoMemSetZero(LLValue *dst, LLValue *nbytes, unsigned align) {
+void DtoMemSetZero(LLType *type, LLValue *dst, LLValue *nbytes, unsigned align) {
   DtoMemSet(dst, DtoConstUbyte(0), nbytes, align);
 }
 
-void DtoMemSetZero(LLValue *dst, unsigned align) {
-  uint64_t n = getTypeStoreSize(dst->getType()->getContainedType(0));
-  DtoMemSetZero(dst, DtoConstSize_t(n), align);
+void DtoMemSetZero(LLType *type, LLValue *dst, unsigned align) {
+  uint64_t n = getTypeStoreSize(type);
+  DtoMemSetZero(type, dst, DtoConstSize_t(n), align);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -418,14 +432,13 @@ void DtoMemCpy(LLValue *dst, LLValue *src, LLValue *nbytes, unsigned align) {
   dst = DtoBitCast(dst, VoidPtrTy);
   src = DtoBitCast(src, VoidPtrTy);
 
-  auto A = LLMaybeAlign(align);
+  auto A = llvm::MaybeAlign(align);
   gIR->ir->CreateMemCpy(dst, A, src, A, nbytes, false /*isVolatile*/);
 }
 
-void DtoMemCpy(LLValue *dst, LLValue *src, bool withPadding, unsigned align) {
-  LLType *pointee = dst->getType()->getContainedType(0);
+void DtoMemCpy(LLType *type, LLValue *dst, LLValue *src, bool withPadding, unsigned align) {
   uint64_t n =
-      withPadding ? getTypeAllocSize(pointee) : getTypeStoreSize(pointee);
+      withPadding ? getTypeAllocSize(type) : getTypeStoreSize(type);
   DtoMemCpy(dst, src, DtoConstSize_t(n), align);
 }
 
@@ -455,6 +468,12 @@ LLValue *DtoMemCmp(LLValue *lhs, LLValue *rhs, LLValue *nbytes) {
 llvm::ConstantInt *DtoConstSize_t(uint64_t i) {
   return LLConstantInt::get(DtoSize_t(), i, false);
 }
+llvm::ConstantInt *DtoConstUlong(uint64_t i) {
+  return LLConstantInt::get(LLType::getInt64Ty(gIR->context()), i, false);
+}
+llvm::ConstantInt *DtoConstLong(int64_t i) {
+  return LLConstantInt::get(LLType::getInt64Ty(gIR->context()), i, true);
+}
 llvm::ConstantInt *DtoConstUint(unsigned i) {
   return LLConstantInt::get(LLType::getInt32Ty(gIR->context()), i, false);
 }
@@ -491,7 +510,7 @@ LLConstant *DtoConstFP(Type *t, const real_t value) {
 LLConstant *DtoConstCString(const char *str) {
   llvm::StringRef s(str ? str : "");
   LLGlobalVariable *gvar = gIR->getCachedStringLiteral(s);
-  return DtoGEP(gvar, 0u, 0u);
+  return DtoGEP(gvar->getValueType(), gvar, 0u, 0u);
 }
 
 LLConstant *DtoConstString(const char *str) {
@@ -503,27 +522,29 @@ LLConstant *DtoConstString(const char *str) {
 ////////////////////////////////////////////////////////////////////////////////
 
 namespace {
-llvm::LoadInst *DtoLoadImpl(LLValue *src, const char *name) {
-  return gIR->ir->CreateLoad(getPointeeType(src), src, name);
+llvm::LoadInst *DtoLoadImpl(LLType *type, LLValue *src, const char *name) {
+  return gIR->ir->CreateLoad(type, src, name);
 }
 }
 
-LLValue *DtoLoad(LLValue *src, const char *name) {
-  return DtoLoadImpl(src, name);
+LLValue *DtoLoad(LLType* type, LLValue *src, const char *name) {
+  return DtoLoadImpl(type, src, name);
+}
+
+LLValue *DtoLoad(DLValue *src, const char *name) {
+  return DtoLoadImpl(DtoType(src->type), DtoLVal(src), name);
 }
 
 // Like DtoLoad, but the pointer is guaranteed to be aligned appropriately for
 // the type.
-LLValue *DtoAlignedLoad(LLValue *src, const char *name) {
-  llvm::LoadInst *ld = DtoLoadImpl(src, name);
-  if (auto alignment = getABITypeAlign(ld->getType())) {
-    ld->setAlignment(LLAlign(alignment));
-  }
+LLValue *DtoAlignedLoad(LLType *type, LLValue *src, const char *name) {
+  llvm::LoadInst *ld = DtoLoadImpl(type, src, name);
+  ld->setAlignment(gDataLayout->getABITypeAlign(ld->getType()));
   return ld;
 }
 
-LLValue *DtoVolatileLoad(LLValue *src, const char *name) {
-  llvm::LoadInst *ld = DtoLoadImpl(src, name);
+LLValue *DtoVolatileLoad(LLType *type, LLValue *src, const char *name) {
+  llvm::LoadInst *ld = DtoLoadImpl(type, src, name);
   ld->setVolatile(true);
   return ld;
 }
@@ -543,7 +564,6 @@ void DtoVolatileStore(LLValue *src, LLValue *dst) {
 void DtoStoreZextI8(LLValue *src, LLValue *dst) {
   if (src->getType()->isIntegerTy(1)) {
     llvm::Type *i8 = llvm::Type::getInt8Ty(gIR->context());
-    assert(dst->getType()->getContainedType(0) == i8);
     src = gIR->ir->CreateZExt(src, i8);
   }
   gIR->ir->CreateStore(src, dst);
@@ -555,9 +575,7 @@ void DtoAlignedStore(LLValue *src, LLValue *dst) {
   assert(!src->getType()->isIntegerTy(1) &&
          "Should store bools as i8 instead of i1.");
   llvm::StoreInst *st = gIR->ir->CreateStore(src, dst);
-  if (auto alignment = getABITypeAlign(src->getType())) {
-    st->setAlignment(LLAlign(alignment));
-  }
+  st->setAlignment(gDataLayout->getABITypeAlign(src->getType()));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -568,15 +586,39 @@ LLType *stripAddrSpaces(LLType *t)
   if(gIR->dcomputetarget == nullptr)
     return t;
 
+  llvm::PointerType *pt = isaPointer(t);
+  if (!pt)
+    return t;
+
+#if LDC_LLVM_VER >= 1400
+  if (pt->isOpaque())
+    return getVoidPtrType();
+  else {
+    int indirections = 0;
+    while (t->isPointerTy()) {
+      indirections++;
+// Disable [[deprecated]] warning on getPointerElementType. We solved the
+// deprecation for versions >= LLVM 16 above (8 lines up).
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+      t = t->getPointerElementType();
+#pragma GCC diagnostic pop
+    }
+    while (indirections-- != 0)
+      t = t->getPointerTo(0);
+  }
+  return t;
+#else
   int indirections = 0;
   while (t->isPointerTy()) {
     indirections++;
     t = t->getPointerElementType();
   }
   while (indirections-- != 0)
-     t = t->getPointerTo(0);
+    t = t->getPointerTo(0);
 
   return t;
+#endif
 }
 
 LLValue *DtoBitCast(LLValue *v, LLType *t, const llvm::Twine &name) {
@@ -680,6 +722,8 @@ llvm::GlobalVariable *isaGlobalVar(LLValue *v) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
+LLType *getI8Type() { return LLType::getInt8Ty(gIR->context()); }
+
 LLPointerType *getPtrToType(LLType *t) {
   if (t == LLType::getVoidTy(gIR->context()))
     t = LLType::getInt8Ty(gIR->context());
@@ -706,7 +750,7 @@ size_t getTypeStoreSize(LLType *t) { return gDataLayout->getTypeStoreSize(t); }
 size_t getTypeAllocSize(LLType *t) { return gDataLayout->getTypeAllocSize(t); }
 
 unsigned int getABITypeAlign(LLType *t) {
-  return gDataLayout->getABITypeAlignment(t);
+  return gDataLayout->getABITypeAlign(t).value();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -744,7 +788,7 @@ LLValue *DtoAggrPair(LLValue *V1, LLValue *V2, const char *name) {
   return DtoAggrPair(t, V1, V2, name);
 }
 
-LLValue *DtoAggrPaint(LLValue *aggr, LLType *as) {
+LLValue *DtoSlicePaint(LLValue *aggr, LLType *as) {
   if (aggr->getType() == as) {
     return aggr;
   }
