@@ -57,14 +57,16 @@
 #include <cassert>
 #include <cstdio>
 
-void genTypeInfo(const Loc &loc, Type *torig, Scope *sc); // in dmd/typinf.d
+// in dmd/typinf.d:
+void genTypeInfo(Expression *e, const Loc &loc, Type *torig, Scope *sc, bool genObjCode = true);
 
 TypeInfoDeclaration *getOrCreateTypeInfoDeclaration(const Loc &loc, Type *forType) {
   IF_LOG Logger::println("getOrCreateTypeInfoDeclaration(): %s",
                          forType->toChars());
   LOG_SCOPE
 
-  genTypeInfo(loc, forType, nullptr);
+  // the `genObjCode` parameter is unused by LDC
+  genTypeInfo(nullptr, loc, forType, nullptr, false);
 
   return forType->vtinfo;
 }
@@ -93,6 +95,11 @@ void emitTypeInfoMetadata(LLGlobalVariable *typeinfoGlobal, Type *forType) {
       auto val = llvm::UndefValue::get(DtoType(forType));
       meta->addOperand(llvm::MDNode::get(gIR->context(),
                                          llvm::ConstantAsMetadata::get(val)));
+      if (TypeArray *ta = t->isTypeDArray()) {
+        auto val2 = llvm::UndefValue::get(DtoMemType(ta->nextOf()));
+        meta->addOperand(llvm::MDNode::get(gIR->context(),
+                                           llvm::ConstantAsMetadata::get(val2)));
+      }
     }
   }
 }
@@ -435,7 +442,8 @@ void buildTypeInfo(TypeInfoDeclaration *decl) {
     assert(isBuiltin && "existing global expected to be the init symbol of a "
                         "built-in TypeInfo");
   } else {
-    LLType *type = DtoType(decl->type)->getPointerElementType();
+    TypeClass *tc = decl->type->isTypeClass();
+    LLType *type = getIrType(tc->sym->type, true)->isClass()->getMemoryLLType();
     // We need to keep the symbol mutable as the type is not declared as
     // immutable on the D side, and e.g. synchronized() can be used on the
     // implicit monitor.
