@@ -17,15 +17,6 @@
 #include "llvm/ADT/StringExtras.h"
 #include "llvm/ADT/StringSwitch.h"
 
-#if LDC_LLVM_VER < 1100
-namespace llvm {
-// Auto-generate:
-// Attribute::AttrKind getAttrKindFromName(StringRef AttrName) { ... }
-#define GET_ATTR_KIND_FROM_NAME
-#include "llvm/IR/Attributes.inc"
-}
-#endif
-
 namespace {
 
 /// Checks whether `moduleDecl` is in the ldc package and it's identifier is
@@ -95,11 +86,11 @@ void checkStructElems(StructLiteralExp *sle, ArrayParam<Type *> elemTypes) {
 /// if it is applied to `sym`, otherwise returns nullptr.
 StructLiteralExp *getMagicAttribute(Dsymbol *sym, const Identifier *id,
                                     const Identifier *from) {
-  if (!sym->userAttribDecl)
+  if (!sym->userAttribDecl())
     return nullptr;
 
   // Loop over all UDAs and early return the expression if a match was found.
-  Expressions *attrs = sym->userAttribDecl->getAttributes();
+  Expressions *attrs = sym->userAttribDecl()->getAttributes();
   expandTuples(attrs);
   for (auto attr : *attrs) {
     if (auto sle = attr->isStructLiteralExp())
@@ -110,17 +101,38 @@ StructLiteralExp *getMagicAttribute(Dsymbol *sym, const Identifier *id,
   return nullptr;
 }
 
+/// Returns the _last_ StructLiteralExp magic attribute with identifier `id`
+/// from the ldc magic module with identifier `from` (attributes or dcompute) if
+/// it is applied to `sym`, otherwise returns nullptr.
+StructLiteralExp *getLastMagicAttribute(Dsymbol *sym, const Identifier *id,
+                                        const Identifier *from) {
+  if (!sym->userAttribDecl())
+    return nullptr;
+
+  // Loop over all UDAs and find the last match
+  StructLiteralExp *lastMatch = nullptr;
+  Expressions *attrs = sym->userAttribDecl()->getAttributes();
+  expandTuples(attrs);
+  for (auto attr : *attrs) {
+    if (auto sle = attr->isStructLiteralExp())
+      if (isFromMagicModule(sle, from) && id == sle->sd->ident)
+        lastMatch = sle;
+  }
+
+  return lastMatch;
+}
+
 /// Calls `action` for each magic attribute with identifier `id` from
 /// the ldc magic module with identifier `from` (attributes or dcompute)
 /// applied to `sym`.
 void callForEachMagicAttribute(Dsymbol &sym, const Identifier *id,
                                const Identifier *from,
                                std::function<void(StructLiteralExp *)> action) {
-  if (!sym.userAttribDecl)
+  if (!sym.userAttribDecl())
     return;
 
   // Loop over all UDAs and call `action` if a match was found.
-  Expressions *attrs = sym.userAttribDecl->getAttributes();
+  Expressions *attrs = sym.userAttribDecl()->getAttributes();
   expandTuples(attrs);
   for (auto attr : *attrs) {
     if (auto sle = attr->isStructLiteralExp())
@@ -198,7 +210,11 @@ void applyAttrAllocSize(StructLiteralExp *sle, IrFunction *irFunc) {
   if (numArgIdx >= 0) {
     builder.addAllocSizeAttr(llvmSizeIdx, llvmNumIdx);
   } else {
+#if LDC_LLVM_VER < 1600
     builder.addAllocSizeAttr(llvmSizeIdx, llvm::Optional<unsigned>());
+#else
+    builder.addAllocSizeAttr(llvmSizeIdx, std::optional<unsigned>());
+#endif
   }
 
   llvm::Function *func = irFunc->getLLVMFunc();
@@ -217,11 +233,7 @@ void applyAttrLLVMAttr(StructLiteralExp *sle, llvm::AttrBuilder &attrs) {
   llvm::StringRef key = getStringElem(sle, 0);
   llvm::StringRef value = getStringElem(sle, 1);
   if (value.empty()) {
-#if LDC_LLVM_VER >= 1100
     const auto kind = llvm::Attribute::getAttrKindFromName(key);
-#else
-    const auto kind = llvm::getAttrKindFromName(key);
-#endif
     if (kind != llvm::Attribute::None) {
       attrs.addAttribute(kind);
     } else {
@@ -366,13 +378,106 @@ void applyAttrAssumeUsed(IRState &irs, StructLiteralExp *sle,
   irs.usedArray.push_back(symbol);
 }
 
+/// Tries to recognize the calling convention,
+/// TODO: Add support "cc <n>"
+/// Returns true if succesful, with the calling convention in 'callconv'.
+/// Returns false if unsuccesful.
+bool parseCallingConvention(llvm::StringRef name,
+                            llvm::CallingConv::ID *callconv) {
+
+  llvm::CallingConv::ID conv_id =
+      llvm::StringSwitch<llvm::CallingConv::ID>(name)
+          // Names recognized by Clang (see Clang's
+          // CodeGenTypes::ClangCallConvToLLVMCallConv):
+          .Case("stdcall", llvm::CallingConv::X86_StdCall)
+          .Case("fastcall", llvm::CallingConv::X86_FastCall)
+          .Case("regcall", llvm::CallingConv::X86_RegCall)
+          .Case("thiscall", llvm::CallingConv::X86_ThisCall)
+          .Case("ms_abi", llvm::CallingConv::Win64)
+          .Case("sysv_abi", llvm::CallingConv::X86_64_SysV)
+          .Case("pcs(\"aapcs\")", llvm::CallingConv::ARM_AAPCS)
+          .Case("pcs(\"aapcs-vfp\")", llvm::CallingConv::ARM_AAPCS_VFP)
+          .Case("intel_ocl_bicc", llvm::CallingConv::Intel_OCL_BI)
+          .Case("pascal",
+                llvm::CallingConv::C) // Check Clang, this may change in future
+          .Case("vectorcall", llvm::CallingConv::X86_VectorCall)
+          .Case("aarch64_vector_pcs", llvm::CallingConv::AArch64_VectorCall)
+          .Case("preserve_most", llvm::CallingConv::PreserveMost)
+          .Case("preserve_all", llvm::CallingConv::PreserveAll)
+          .Case("swiftcall", llvm::CallingConv::Swift)
+#if LDC_LLVM_VER >= 1300
+          .Case("swiftasynccall", llvm::CallingConv::SwiftTail)
+#endif
+
+          // Names recognized in LLVM IR (see LLVM's
+          // LLParser::parseOptionalCallingConv):
+          .Case("ccc", llvm::CallingConv::C)
+          .Case("fastcc", llvm::CallingConv::Fast)
+          .Case("coldcc", llvm::CallingConv::Cold)
+          .Case("cfguard_checkcc", llvm::CallingConv::CFGuard_Check)
+          .Case("x86_stdcallcc", llvm::CallingConv::X86_StdCall)
+          .Case("x86_fastcallcc", llvm::CallingConv::X86_FastCall)
+          .Case("x86_regcallcc", llvm::CallingConv::X86_RegCall)
+          .Case("x86_thiscallcc", llvm::CallingConv::X86_ThisCall)
+          .Case("x86_vectorcallcc", llvm::CallingConv::X86_VectorCall)
+          .Case("arm_apcscc", llvm::CallingConv::ARM_APCS)
+          .Case("arm_aapcscc", llvm::CallingConv::ARM_AAPCS)
+          .Case("arm_aapcs_vfpcc", llvm::CallingConv::ARM_AAPCS_VFP)
+          .Case("aarch64_vector_pcs", llvm::CallingConv::AArch64_VectorCall)
+          .Case("aarch64_sve_vector_pcs",
+                llvm::CallingConv::AArch64_SVE_VectorCall)
+          .Case("msp430_intrcc", llvm::CallingConv::MSP430_INTR)
+          .Case("avr_intrcc", llvm::CallingConv::AVR_INTR)
+          .Case("avr_signalcc", llvm::CallingConv::AVR_SIGNAL)
+          .Case("ptx_kernel", llvm::CallingConv::PTX_Kernel)
+          .Case("ptx_device", llvm::CallingConv::PTX_Device)
+          .Case("spir_kernel", llvm::CallingConv::SPIR_KERNEL)
+          .Case("spir_func", llvm::CallingConv::SPIR_FUNC)
+          .Case("intel_ocl_bicc", llvm::CallingConv::Intel_OCL_BI)
+          .Case("x86_64_sysvcc", llvm::CallingConv::X86_64_SysV)
+          .Case("win64cc", llvm::CallingConv::Win64)
+          .Case("webkit_jscc", llvm::CallingConv::WebKit_JS)
+          .Case("anyregcc", llvm::CallingConv::AnyReg)
+          .Case("preserve_mostcc", llvm::CallingConv::PreserveMost)
+          .Case("preserve_allcc", llvm::CallingConv::PreserveAll)
+          .Case("ghccc", llvm::CallingConv::GHC)
+          .Case("swiftcc", llvm::CallingConv::Swift)
+#if LDC_LLVM_VER >= 1300
+          .Case("swifttailcc", llvm::CallingConv::SwiftTail)
+#endif
+          .Case("x86_intrcc", llvm::CallingConv::X86_INTR)
+          .Case("hhvmcc", llvm::CallingConv::HHVM)
+          .Case("hhvm_ccc", llvm::CallingConv::HHVM_C)
+          .Case("cxx_fast_tlscc", llvm::CallingConv::CXX_FAST_TLS)
+          .Case("amdgpu_vs", llvm::CallingConv::AMDGPU_VS)
+#if LDC_LLVM_VER >= 1200
+          .Case("amdgpu_gfx", llvm::CallingConv::AMDGPU_Gfx)
+#endif
+          .Case("amdgpu_ls", llvm::CallingConv::AMDGPU_LS)
+          .Case("amdgpu_hs", llvm::CallingConv::AMDGPU_HS)
+          .Case("amdgpu_es", llvm::CallingConv::AMDGPU_ES)
+          .Case("amdgpu_gs", llvm::CallingConv::AMDGPU_GS)
+          .Case("amdgpu_ps", llvm::CallingConv::AMDGPU_PS)
+          .Case("amdgpu_cs", llvm::CallingConv::AMDGPU_CS)
+          .Case("amdgpu_kernel", llvm::CallingConv::AMDGPU_KERNEL)
+          .Case("tailcc", llvm::CallingConv::Tail)
+
+          .Case("default", llvm::CallingConv::MaxID - 1)
+          .Default(llvm::CallingConv::MaxID);
+
+  bool success = (conv_id != llvm::CallingConv::MaxID);
+  if (success && callconv)
+    *callconv = conv_id;
+  return success;
+}
+
 } // anonymous namespace
 
 void applyVarDeclUDAs(VarDeclaration *decl, llvm::GlobalVariable *gvar) {
-  if (!decl->userAttribDecl)
+  if (!decl->userAttribDecl())
     return;
 
-  Expressions *attrs = decl->userAttribDecl->getAttributes();
+  Expressions *attrs = decl->userAttribDecl()->getAttributes();
   expandTuples(attrs);
   for (auto &attr : *attrs) {
     auto sle = getLdcAttributesStruct(attr);
@@ -394,7 +499,8 @@ void applyVarDeclUDAs(VarDeclaration *decl, llvm::GlobalVariable *gvar) {
     } else if (ident == Id::udaWeak) {
       // @weak is applied elsewhere
     } else if (ident == Id::udaDynamicCompile ||
-               ident == Id::udaDynamicCompileEmit) {
+               ident == Id::udaDynamicCompileEmit ||
+               ident == Id::udaCallingConvention) {
       sle->error(
           "Special attribute `ldc.attributes.%s` is only valid for functions",
           ident->toChars());
@@ -410,11 +516,11 @@ void applyVarDeclUDAs(VarDeclaration *decl, llvm::GlobalVariable *gvar) {
 
 void applyFuncDeclUDAs(FuncDeclaration *decl, IrFunction *irFunc) {
   // function UDAs
-  if (decl->userAttribDecl) {
+  if (decl->userAttribDecl()) {
     llvm::Function *func = irFunc->getLLVMFunc();
     assert(func);
 
-    Expressions *attrs = decl->userAttribDecl->getAttributes();
+    Expressions *attrs = decl->userAttribDecl()->getAttributes();
     expandTuples(attrs);
     for (auto &attr : *attrs) {
       auto sle = getLdcAttributesStruct(attr);
@@ -450,7 +556,9 @@ void applyFuncDeclUDAs(FuncDeclaration *decl, IrFunction *irFunc) {
       } else if (ident == Id::udaAssumeUsed) {
         applyAttrAssumeUsed(*gIR, sle, func);
       } else if (ident == Id::udaWeak || ident == Id::udaKernel ||
-                 ident == Id::udaNoSanitize) {
+                 ident == Id::udaNoSanitize ||
+                 ident == Id::udaCallingConvention ||
+                 ident == Id::udaNoSplitStack) {
         // These UDAs are applied elsewhere, thus should silently be ignored here.
       } else if (ident == Id::udaDynamicCompile) {
         irFunc->dynamicCompile = true;
@@ -498,6 +606,25 @@ void applyFuncDeclUDAs(FuncDeclaration *decl, IrFunction *irFunc) {
   }
 }
 
+/// Checks whether 'fd' has the @ldc.attributes.callingConvention("...") UDA applied.
+/// If so, it returns the calling convention in 'callconv'.
+bool hasCallingConventionUDA(FuncDeclaration *fd,
+                             llvm::CallingConv::ID *callconv) {
+  auto sle =
+      getLastMagicAttribute(fd, Id::udaCallingConvention, Id::attributes);
+  if (!sle)
+    return false;
+
+  checkStructElems(sle, {Type::tstring});
+  auto name = getFirstElemString(sle);
+  bool success = parseCallingConvention(name, callconv);
+  if (!success)
+    sle->warning("Ignoring unrecognized calling convention name '%s' for "
+                 "`@ldc.attributes.callingConvention`",
+                 name.str().c_str());
+  return success;
+}
+
 /// Checks whether 'sym' has the @ldc.attributes._weak() UDA applied.
 bool hasWeakUDA(Dsymbol *sym) {
   auto sle = getMagicAttribute(sym, Id::udaWeak, Id::attributes);
@@ -542,6 +669,12 @@ bool hasKernelAttr(Dsymbol *sym) {
   return true;
 }
 
+/// Check whether `fd` has the `@ldc.attributes.noSplitStack` UDA applied.
+bool hasNoSplitStackUDA(FuncDeclaration *fd) {
+  auto sle = getMagicAttribute(fd, Id::udaNoSplitStack, Id::attributes);
+  return sle != nullptr;
+}
+
 /// Creates a mask (for &) of @ldc.attributes.noSanitize UDA applied to the
 /// function.
 /// If a bit is set in the mask, then the sanitizer is enabled.
