@@ -15,7 +15,7 @@
 #include "dmd/mangle.h"
 #include "dmd/mtype.h"
 #include "dmd/target.h"
-#include "gen/abi.h"
+#include "gen/abi/abi.h"
 #include "gen/arrays.h"
 #include "gen/funcgenstate.h"
 #include "gen/functions.h"
@@ -110,8 +110,7 @@ LLGlobalVariable *IrClass::getClassInfoSymbol(bool define) {
       emitTypeInfoMetadata(typeInfo, aggrdecl->type);
 
       // Gather information
-      LLType *type = DtoType(aggrdecl->type);
-      LLType *bodyType = type->getPointerElementType();
+      LLType *bodyType = getLLStructType();
       bool hasDestructor = (aggrdecl->dtor != nullptr);
       // Construct the fields
       llvm::Metadata *mdVals[CD_NumFields];
@@ -122,8 +121,7 @@ LLGlobalVariable *IrClass::getClassInfoSymbol(bool define) {
       // Construct the metadata and insert it into the module.
       const auto metaname = getMetadataName(CD_PREFIX, typeInfo);
       llvm::NamedMDNode *node = gIR->module.getOrInsertNamedMetadata(metaname);
-      node->addOperand(llvm::MDNode::get(
-          gIR->context(), llvm::makeArrayRef(mdVals, CD_NumFields)));
+      node->addOperand(llvm::MDNode::get(gIR->context(), mdVals));
     }
 
     if (!define)
@@ -247,8 +245,8 @@ LLConstant *IrClass::getVtblInit() {
           if (fd2->isFuture()) {
             continue;
           }
-          if (fd->leastAsSpecialized(fd2) != MATCH::nomatch ||
-              fd2->leastAsSpecialized(fd) != MATCH::nomatch) {
+          if (fd->leastAsSpecialized(fd2, nullptr) != MATCH::nomatch ||
+              fd2->leastAsSpecialized(fd, nullptr) != MATCH::nomatch) {
             TypeFunction *tf = static_cast<TypeFunction *>(fd->type);
             if (tf->ty == TY::Tfunction) {
               cd->error("use of `%s%s` is hidden by `%s`; use `alias %s = "
@@ -427,7 +425,7 @@ LLConstant *IrClass::getClassInfoInit() {
   }
 
   // build the initializer
-  LLType *initType = getClassInfoSymbol()->getType()->getContainedType(0);
+  LLType *initType = getClassInfoSymbol()->getValueType();
   constTypeInfo = b.get_constant(isaStruct(initType));
 
   return constTypeInfo;
@@ -449,9 +447,9 @@ llvm::GlobalVariable *IrClass::getInterfaceVtblSymbol(BaseClass *b,
 
     // Thunk prefix
     char thunkPrefix[16];
-    int thunkLen = sprintf(thunkPrefix, "Thn%d_", b->offset);
+    int thunkLen = snprintf(thunkPrefix, 16, "Thn%d_", b->offset);
     char thunkPrefixLen[16];
-    sprintf(thunkPrefixLen, "%d", thunkLen);
+    snprintf(thunkPrefixLen, 16, "%d", thunkLen);
 
     OutBuffer mangledName;
     mangledName.writestring("_D");
@@ -500,7 +498,7 @@ LLConstant *IrClass::getInterfaceVtblInit(BaseClass *b,
   constants.reserve(vtbl_array.length);
 
   char thunkPrefix[16];
-  sprintf(thunkPrefix, "Thn%d_", b->offset);
+  snprintf(thunkPrefix, 16, "Thn%d_", b->offset);
 
   const auto voidPtrTy = getVoidPtrType();
 
@@ -512,7 +510,7 @@ LLConstant *IrClass::getInterfaceVtblInit(BaseClass *b,
 
       llvm::GlobalVariable *interfaceInfosZ = getInterfaceArraySymbol();
       llvm::Constant *c = llvm::ConstantExpr::getGetElementPtr(
-          getPointeeType(interfaceInfosZ), interfaceInfosZ, idxs, true);
+            interfaceInfosZ->getValueType(), interfaceInfosZ, idxs, true);
 
       constants.push_back(DtoBitCast(c, voidPtrTy));
     } else {
@@ -568,7 +566,7 @@ LLConstant *IrClass::getInterfaceVtblInit(BaseClass *b,
                                   needsCOMDAT());
       const auto callee = irFunc->getLLVMCallee();
       thunk = LLFunction::Create(
-          isaFunction(callee->getType()->getContainedType(0)), lwc.first,
+          callee->getFunctionType(), lwc.first,
           thunkIRMangle, &gIR->module);
       setLinkage(lwc, thunk);
       thunk->copyAttributesFrom(callee);
@@ -626,7 +624,7 @@ LLConstant *IrClass::getInterfaceVtblInit(BaseClass *b,
       LLValue *&thisArg = args[thisArgIndex];
       LLType *targetThisType = thisArg->getType();
       thisArg = DtoBitCast(thisArg, getVoidPtrType());
-      thisArg = DtoGEP1(thisArg, DtoConstInt(-thunkOffset));
+      thisArg = DtoGEP1(LLType::getInt8Ty(gIR->context()), thisArg, DtoConstInt(-thunkOffset));
       thisArg = DtoBitCast(thisArg, targetThisType);
 
       // all calls that might be subject to inlining into a caller with debug
@@ -739,8 +737,7 @@ LLConstant *IrClass::getClassInfoInterfaces() {
 
     // create Interface struct
     LLConstant *inits[3] = {ci, vtb, off};
-    LLConstant *entry =
-        LLConstantStruct::get(interface_type, llvm::makeArrayRef(inits, 3));
+    LLConstant *entry = LLConstantStruct::get(interface_type, inits);
     constants.push_back(entry);
   }
 
@@ -762,7 +759,7 @@ LLConstant *IrClass::getClassInfoInterfaces() {
   LLConstant *idxs[2] = {DtoConstSize_t(0),
                          DtoConstSize_t(n - cd->vtblInterfaces->length)};
 
-  LLConstant *ptr = llvm::ConstantExpr::getGetElementPtr(getPointeeType(ciarr),
+  LLConstant *ptr = llvm::ConstantExpr::getGetElementPtr(ciarr->getValueType(),
                                                          ciarr, idxs, true);
 
   // return as a slice
